@@ -110,22 +110,15 @@
     (when (plusp count)
       (format t "~&Removed ~D doc string~:P" count)))
 
-  ;; Share identical FUN-INFOs
-  sb-int::
-  (let ((ht (make-hash-table :test 'equalp))
-        (old-count 0))
-    (sb-int:call-with-each-globaldb-name
-     (lambda (name)
-       (binding* ((info (info :function :info name) :exit-if-null)
-                  (shared-info (gethash info ht info)))
-         (incf old-count)
-         (if (eq info shared-info)
-             (setf (gethash info ht) info)
-           (setf (info :function :info name) shared-info)))))
-    (format t "~&FUN-INFO: Collapsed ~D -> ~D~%"
-            old-count (hash-table-count ht)))
-
-  (sb-disassem::!compile-inst-printers)
+  ;; Remove source forms of compiled-to-memory lambda expressions.
+  ;; The disassembler is the major culprit for retention of these.
+  (sb-vm::map-allocated-objects
+   (lambda (obj type size)
+     (declare (ignore size))
+     (when (typep obj 'sb-c::debug-source)
+       (unless (sb-c::debug-source-namestring obj)
+         (setf (sb-c::debug-source-form obj) nil))))
+   :all)
 
   ;; Unintern no-longer-needed stuff before the possible PURIFY in
   ;; SAVE-LISP-AND-DIE.
@@ -212,6 +205,10 @@
          (and (boundp symbol) (not (keywordp symbol))))))
   (unintern 'sb-impl::shake-packages 'sb-impl))
 
+;;; Use historical (stupid) behavior for storing pathname namestrings
+;;; in fasls.
+(setq sb-c::*name-context-file-path-selector* 'truename)
+
 ;;; Lock internal packages
 #+sb-package-locks
 (dolist (p (list-all-packages))
