@@ -101,7 +101,6 @@
 ;;;; power functions
 
 (defun exp (number)
-  #!+sb-doc
   "Return e raised to the power NUMBER."
   (declare (explicit-check))
   (number-dispatch ((number number))
@@ -156,7 +155,6 @@
 ;;; result. We also separate the complex-real and real-complex cases
 ;;; from the general complex case.
 (defun expt (base power)
-  #!+sb-doc
   "Return BASE raised to the POWER."
   (declare (explicit-check))
   (if (zerop power)
@@ -164,7 +162,7 @@
         (error 'arguments-out-of-domain-error
                :operands (list base power)
                :operation 'expt
-               :references (list '(:ansi-cl :function expt)))
+               :references '((:ansi-cl :function expt)))
         (let ((result (1+ (* base power))))
           (if (and (floatp result) (float-nan-p result))
               (float 1 result)
@@ -359,7 +357,6 @@
                     2.0d0))))))
 
 (defun log (number &optional (base nil base-p))
-  #!+sb-doc
   "Return the logarithm of NUMBER in the base BASE, which defaults to e."
   (declare (explicit-check))
   (if base-p
@@ -408,7 +405,6 @@
          (complex-log number)))))
 
 (defun sqrt (number)
-  #!+sb-doc
   "Return the square root of NUMBER."
   (declare (explicit-check))
   (number-dispatch ((number number))
@@ -430,7 +426,6 @@
 ;;;; trigonometic and related functions
 
 (defun abs (number)
-  #!+sb-doc
   "Return the absolute value of the number."
   (declare (explicit-check))
   (number-dispatch ((number number))
@@ -450,7 +445,6 @@
           (%hypot rx (truly-the double-float ix))))))))
 
 (defun phase (number)
-  #!+sb-doc
   "Return the angle part of the polar representation of a complex number.
   For complex numbers, this is (atan (imagpart number) (realpart number)).
   For non-complex positive numbers, this is 0. For non-complex negative
@@ -473,7 +467,6 @@
      (atan (imagpart number) (realpart number)))))
 
 (defun sin (number)
-  #!+sb-doc
   "Return the sine of NUMBER."
   (declare (explicit-check))
   (number-dispatch ((number number))
@@ -485,7 +478,6 @@
                 (* (cos x) (sinh y)))))))
 
 (defun cos (number)
-  #!+sb-doc
   "Return the cosine of NUMBER."
   (declare (explicit-check))
   (number-dispatch ((number number))
@@ -497,7 +489,6 @@
                 (- (* (sin x) (sinh y))))))))
 
 (defun tan (number)
-  #!+sb-doc
   "Return the tangent of NUMBER."
   (declare (explicit-check))
   (number-dispatch ((number number))
@@ -510,7 +501,6 @@
                 (- (realpart result)))))))
 
 (defun cis (theta)
-  #!+sb-doc
   "Return cos(Theta) + i sin(Theta), i.e. exp(i Theta)."
   (declare (explicit-check ))
   (number-dispatch ((theta real))
@@ -518,7 +508,6 @@
      (complex (cos theta) (sin theta)))))
 
 (defun asin (number)
-  #!+sb-doc
   "Return the arc sine of NUMBER."
   (declare (explicit-check))
   (number-dispatch ((number number))
@@ -536,7 +525,6 @@
      (complex-asin number))))
 
 (defun acos (number)
-  #!+sb-doc
   "Return the arc cosine of NUMBER."
   (declare (explicit-check))
   (number-dispatch ((number number))
@@ -554,7 +542,6 @@
      (complex-acos number))))
 
 (defun atan (y &optional (x nil xp))
-  #!+sb-doc
   "Return the arc tangent of Y if X is omitted or Y/X if X is supplied."
   (declare (explicit-check))
   (if xp
@@ -591,7 +578,6 @@
 ;;; complex numbers can also lose big.
 
 (defun sinh (number)
-  #!+sb-doc
   "Return the hyperbolic sine of NUMBER."
   (declare (explicit-check))
   (number-dispatch ((number number))
@@ -603,7 +589,6 @@
                 (* (cosh x) (sin y)))))))
 
 (defun cosh (number)
-  #!+sb-doc
   "Return the hyperbolic cosine of NUMBER."
   (declare (explicit-check))
   (number-dispatch ((number number))
@@ -615,7 +600,6 @@
                 (* (sinh x) (sin y)))))))
 
 (defun tanh (number)
-  #!+sb-doc
   "Return the hyperbolic tangent of NUMBER."
   (declare (explicit-check))
   (number-dispatch ((number number))
@@ -624,7 +608,6 @@
      (complex-tanh number))))
 
 (defun asinh (number)
-  #!+sb-doc
   "Return the hyperbolic arc sine of NUMBER."
   (declare (explicit-check))
   (number-dispatch ((number number))
@@ -633,7 +616,6 @@
      (complex-asinh number))))
 
 (defun acosh (number)
-  #!+sb-doc
   "Return the hyperbolic arc cosine of NUMBER."
   (declare (explicit-check))
   (number-dispatch ((number number))
@@ -651,7 +633,6 @@
      (complex-acosh number))))
 
 (defun atanh (number)
-  #!+sb-doc
   "Return the hyperbolic arc tangent of NUMBER."
   (declare (explicit-check))
   (number-dispatch ((number number))
