@@ -19,7 +19,6 @@
 ;;; builds.
 
 (defmacro with-cas-lock ((place) &body body)
-  #!+sb-doc
   "Runs BODY with interrupts disabled and *CURRENT-THREAD* compare-and-swapped
 into PLACE instead of NIL. PLACE must be a place acceptable to
 COMPARE-AND-SWAP, and must initially hold NIL.
@@ -68,7 +67,6 @@ WITH-CAS-LOCK can be entered recursively."
 
 (define-condition thread-error (error)
   ((thread :reader thread-error-thread :initarg :thread))
-  #!+sb-doc
   (:documentation
    "Conditions of type THREAD-ERROR are signalled when thread operations fail.
 The offending thread is initialized by the :THREAD initialization argument and
@@ -92,7 +90,6 @@ read by the function THREAD-ERROR-THREAD."))
                         (cdr part)))
        (format stream "    ~S~%" start)))))
 
-#!+sb-doc
 (setf
  (fdocumentation 'thread-error-thread 'function)
  "Return the offending thread that the THREAD-ERROR pertains to.")
@@ -112,7 +109,6 @@ read by the function THREAD-ERROR-THREAD."))
                  (:no-tls-value "the symbol has no thread-local value.")
                  (:thread-dead "the thread has exited.")
                  (:invalid-tls-value "the thread-local value is not valid."))))))
-  #!+sb-doc
   (:documentation
    "Signalled when SYMBOL-VALUE-IN-THREAD or its SETF version fails due to eg.
 the symbol not having a thread-local value, or the target thread having
@@ -135,7 +131,6 @@ offending thread using THREAD-ERROR-THREAD."))
                 (format s "In thread ~A, attempt to join the current ~
                            thread."
                         (thread-error-thread c))))))
-  #!+sb-doc
   (:documentation
    "Signalled when joining a thread fails due to abnormal exit of the thread
 to be joined. The offending thread can be accessed using
@@ -149,7 +144,6 @@ THREAD-ERROR-THREAD."))
   (:report (lambda (c s)
              (format s "Interrupt thread failed: thread ~A has exited."
                      (thread-error-thread c))))
-  #!+sb-doc
   (:documentation
    "Signalled when interrupting a thread fails because the thread has already
 exited. The offending thread can be accessed using THREAD-ERROR-THREAD."))
@@ -163,13 +157,6 @@ exited. The offending thread can be accessed using THREAD-ERROR-THREAD."))
 ;;; gencgc and numbers on the stack (returned by GET-LISP-OBJ-ADDRESS)
 ;;; are treated as references.
 
-;;; set the doc here because in early-thread FDOCUMENTATION is not
-;;; available, yet
-#!+sb-doc
-(setf (fdocumentation '*current-thread* 'variable)
-      "Bound in each thread to the thread itself.")
-
-#!+sb-doc
 (setf
  (fdocumentation 'thread-name 'function)
  "Name of the thread. Can be assigned to using SETF. Thread names can be
@@ -218,14 +205,12 @@ arbitrary printable objects, and need not be unique.")
   (print-lock mutex (mutex-name mutex) (mutex-owner mutex) stream))
 
 (defun thread-alive-p (thread)
-  #!+sb-doc
   "Return T if THREAD is still alive. Note that the return value is
 potentially stale even before the function returns, as the thread may exit at
 any time."
   (thread-%alive-p thread))
 
 (defun thread-ephemeral-p (thread)
-  #!+sb-doc
   "Return T if THREAD is `ephemeral', which indicates that this thread is
 used by SBCL for internal purposes, and specifically that it knows how to
 to terminate this thread cleanly prior to core file saving without signalling
@@ -235,8 +220,8 @@ an error in that case."
 ;; A thread is eligible for gc iff it has finished and there are no
 ;; more references to it. This list is supposed to keep a reference to
 ;; all running threads.
-(defvar *all-threads* ())
-(defvar *all-threads-lock* (make-mutex :name "all threads lock"))
+(sb!ext:define-load-time-global *all-threads* ())
+(sb!ext:define-load-time-global *all-threads-lock* (make-mutex :name "all threads lock"))
 
 (defvar *default-alloc-signal* nil)
 
@@ -245,7 +230,6 @@ an error in that case."
      ,@body))
 
 (defun list-all-threads ()
-  #!+sb-doc
   "Return a list of the live threads. Note that the return value is
 potentially stale even before the function returns, as new threads may be
 created and old ones may exit at any time."
@@ -290,8 +274,8 @@ created and old ones may exit at any time."
   #!-sb-thread
   0)
 
-(defvar *initial-thread* nil)
-(defvar *make-thread-lock*)
+(sb!ext:define-load-time-global *initial-thread* nil)
+(sb!ext:define-load-time-global *make-thread-lock* nil)
 
 (defun init-initial-thread ()
   (/show0 "Entering INIT-INITIAL-THREAD")
@@ -309,17 +293,14 @@ created and old ones may exit at any time."
     (setq *all-threads* (list initial-thread))))
 
 (defun main-thread ()
-  #!+sb-doc
   "Returns the main thread of the process."
   *initial-thread*)
 
 (defun main-thread-p (&optional (thread *current-thread*))
-  #!+sb-doc
   "True if THREAD, defaulting to current thread, is the main thread of the process."
   (eq thread *initial-thread*))
 
 (defmacro return-from-thread (values-form &key allow-exit)
-  #!+sb-doc
   "Unwinds from and terminates the current thread, with values from
 VALUES-FORM as the results visible to JOIN-THREAD.
 
@@ -346,7 +327,6 @@ See also: ABORT-THREAD and SB-EXT:EXIT."
            (throw '%return-from-thread (values-list values))))))
 
 (defun abort-thread (&key allow-exit)
-  #!+sb-doc
   "Unwinds from and terminates the current thread abnormally, causing
 JOIN-THREAD on current thread to signal an error unless a
 default-value is provided.
@@ -450,7 +430,6 @@ See also: RETURN-FROM-THREAD and SB-EXT:EXIT."
 
 ;;;; Mutexes
 
-#!+sb-doc
 (setf (fdocumentation 'make-mutex 'function)
       "Create a mutex."
       (fdocumentation 'mutex-name 'function)
@@ -469,7 +448,6 @@ See also: RETURN-FROM-THREAD and SB-EXT:EXIT."
   (defconstant +lock-contested+ 2))
 
 (defun mutex-owner (mutex)
-  #!+sb-doc
   "Current owner of the mutex, NIL if the mutex is free. Naturally,
 this is racy by design (another thread may acquire the mutex after
 this function returns), it is intended for informative purposes. For
@@ -481,16 +459,14 @@ HOLDING-MUTEX-P."
 (sb!ext:defglobal **deadlock-lock** nil)
 
 #!+(or (not sb-thread) sb-futex)
-(defstruct (waitqueue (:constructor make-waitqueue (&key name)))
-  #!+sb-doc
+(defstruct (waitqueue (:copier nil) (:constructor make-waitqueue (&key name)))
   "Waitqueue type."
   (name nil :type (or null thread-name))
   #!+(and sb-thread sb-futex)
   (token nil))
 
 #!+(and sb-thread (not sb-futex))
-(defstruct (waitqueue (:constructor make-waitqueue (&key name)))
-  #!+sb-doc
+(defstruct (waitqueue (:copier nil) (:constructor make-waitqueue (&key name)))
   "Waitqueue type."
   (name nil :type (or null thread-name))
   ;; For WITH-CAS-LOCK: because CONDITION-WAIT must be able to call
@@ -680,7 +656,6 @@ HOLDING-MUTEX-P."
             mutex new-owner timeout (decode-timeout timeout))))))
 
 (defun grab-mutex (mutex &key (waitp t) (timeout nil))
-  #!+sb-doc
   "Acquire MUTEX for the current thread. If WAITP is true (the default) and
 the mutex is not immediately available, sleep until it is available.
 
@@ -724,7 +699,6 @@ Notes:
             mutex self timeout (decode-timeout timeout))))))
 
 (defun release-mutex (mutex &key (if-not-owner :punt))
-  #!+sb-doc
   "Release MUTEX by setting it to NIL. Wake up threads waiting for
 this mutex.
 
@@ -822,7 +796,6 @@ IF-NOT-OWNER is :FORCE)."
   (print-unreadable-object (waitqueue stream :type t :identity t)
     (format stream "~@[~A~]" (waitqueue-name waitqueue))))
 
-#!+sb-doc
 (setf (fdocumentation 'waitqueue-name 'function)
       "The name of the waitqueue. Setfable."
       (fdocumentation 'make-waitqueue 'function)
@@ -945,7 +918,6 @@ IF-NOT-OWNER is :FORCE)."
 (declaim (notinline %condition-wait))
 
 (defun condition-wait (queue mutex &key timeout)
-  #!+sb-doc
   "Atomically release MUTEX and start waiting on QUEUE until another thread
 wakes us up using either CONDITION-NOTIFY or CONDITION-BROADCAST on
 QUEUE, at which point we re-acquire MUTEX and return T.
@@ -993,7 +965,6 @@ associated data:
                         to-sec to-usec stop-sec stop-usec deadlinep)))))
 
 (defun condition-notify (queue &optional (n 1))
-  #!+sb-doc
   "Notify N threads waiting on QUEUE.
 
 IMPORTANT: The same mutex that is used in the corresponding CONDITION-WAIT
@@ -1025,7 +996,6 @@ must be held by this thread during this call."
         (futex-wake (waitqueue-token-address queue) n)))))
 
 (defun condition-broadcast (queue)
-  #!+sb-doc
   "Notify all threads waiting on QUEUE.
 
 IMPORTANT: The same mutex that is used in the corresponding CONDITION-WAIT
@@ -1039,19 +1009,18 @@ must be held by this thread during this call."
 
 ;;;; Semaphores
 
-(defstruct (semaphore (:constructor make-semaphore
+(defstruct (semaphore (:copier nil)
+                      (:constructor make-semaphore
                           (&key name ((:count %count) 0))))
-  #!+sb-doc
   "Semaphore type. The fact that a SEMAPHORE is a STRUCTURE-OBJECT
 should be considered an implementation detail, and may change in the
 future."
-  (name    nil :type (or null thread-name))
+  (name    nil :type (or null thread-name) :read-only t)
   (%count    0 :type (integer 0))
   (waitcount 0 :type sb!vm:word)
-  (mutex (make-mutex))
-  (queue (make-waitqueue)))
+  (mutex (make-mutex :name "semaphore lock") :read-only t)
+  (queue (make-waitqueue) :read-only t))
 
-#!+sb-doc
 (setf (fdocumentation 'semaphore-name 'function)
       "The name of the semaphore INSTANCE. Setfable."
       (fdocumentation 'make-semaphore 'function)
@@ -1059,20 +1028,17 @@ future."
 
 (defstruct (semaphore-notification (:constructor make-semaphore-notification ())
                                    (:copier nil))
-  #!+sb-doc
   "Semaphore notification object. Can be passed to WAIT-ON-SEMAPHORE and
 TRY-SEMAPHORE as the :NOTIFICATION argument. Consequences are undefined if
 multiple threads are using the same notification object in parallel."
   (%status nil :type boolean))
 
-#!+sb-doc
 (setf (fdocumentation 'make-semaphore-notification 'function)
       "Constructor for SEMAPHORE-NOTIFICATION objects. SEMAPHORE-NOTIFICATION-STATUS
 is initially NIL.")
 
 (declaim (inline semaphore-notification-status))
 (defun semaphore-notification-status (semaphore-notification)
-  #!+sb-doc
   "Returns T if a WAIT-ON-SEMAPHORE or TRY-SEMAPHORE using
 SEMAPHORE-NOTIFICATION has succeeded since the notification object was created
 or cleared."
@@ -1081,7 +1047,6 @@ or cleared."
 
 (declaim (inline clear-semaphore-notification))
 (defun clear-semaphore-notification (semaphore-notification)
-  #!+sb-doc
   "Resets the SEMAPHORE-NOTIFICATION object for use with another call to
 WAIT-ON-SEMAPHORE or TRY-SEMAPHORE."
   (barrier (:write)
@@ -1089,7 +1054,6 @@ WAIT-ON-SEMAPHORE or TRY-SEMAPHORE."
 
 (declaim (inline semaphore-count))
 (defun semaphore-count (instance)
-  #!+sb-doc
   "Returns the current count of the semaphore INSTANCE."
   (barrier (:read))
   (semaphore-%count instance))
@@ -1167,7 +1131,6 @@ WAIT-ON-SEMAPHORE or TRY-SEMAPHORE."
                            (or null (integer 0)))
                 wait-on-semaphore))
 (defun wait-on-semaphore (semaphore &key (n 1) timeout notification)
-  #!+sb-doc
   "Decrement the count of SEMAPHORE by N if the count would not be negative.
 
 Else blocks until the semaphore can be decremented. Returns the new count of
@@ -1188,7 +1151,6 @@ decrements the count, the status is set to T."
                            (or null (integer 0)))
                 try-semaphore))
 (defun try-semaphore (semaphore &optional (n 1) notification)
-  #!+sb-doc
   "Try to decrement the count of SEMAPHORE by N. If the count were to
 become negative, punt and return NIL, otherwise return the new count of
 SEMAPHORE.
@@ -1199,7 +1161,6 @@ the status is set to T."
   (%decrement-semaphore semaphore n nil notification 'try-semaphore))
 
 (defun signal-semaphore (semaphore &optional (n 1))
-  #!+sb-doc
   "Increment the count of SEMAPHORE by N. If there are threads waiting
 on this semaphore, then N of them is woken up."
   (declare (type (integer 1) n))
@@ -1214,7 +1175,7 @@ on this semaphore, then N of them is woken up."
 
 ;;;; Job control, independent listeners
 
-(defstruct session
+(defstruct (session (:copier nil))
   (lock (make-mutex :name "session lock"))
   (threads nil)
   (interactive-threads nil)
@@ -1249,11 +1210,14 @@ on this semaphore, then N of them is woken up."
 
 (defun %delete-thread-from-session (thread session)
   (with-session-lock (session)
-    (setf (session-threads session)
-          ;; DELQ never conses, but DELETE does. (FIXME)
-          (delq thread (session-threads session))
-          (session-interactive-threads session)
-          (delq thread (session-interactive-threads session)))))
+    (let ((was-foreground (eq thread (foreground-thread session))))
+      (setf (session-threads session)
+            ;; DELQ never conses, but DELETE does. (FIXME)
+            (delq thread (session-threads session))
+            (session-interactive-threads session)
+            (delq thread (session-interactive-threads session)))
+      (when was-foreground
+        (condition-broadcast (session-interactive-threads-queue session))))))
 
 (defun call-with-new-session (fn)
   (%delete-thread-from-session *current-thread* *session*)
@@ -1280,6 +1244,17 @@ on this semaphore, then N of them is woken up."
     (when *session*
       (%delete-thread-from-session thread *session*))))
 
+(defvar sb!ext:*invoke-debugger-hook* nil
+  "This is either NIL or a designator for a function of two arguments,
+   to be run when the debugger is about to be entered.  The function is
+   run with *INVOKE-DEBUGGER-HOOK* bound to NIL to minimize recursive
+   errors, and receives as arguments the condition that triggered
+   debugger entry and the previous value of *INVOKE-DEBUGGER-HOOK*
+
+   This mechanism is an SBCL extension similar to the standard *DEBUGGER-HOOK*.
+   In contrast to *DEBUGGER-HOOK*, it is observed by INVOKE-DEBUGGER even when
+   called by BREAK.")
+
 (defun %exit-other-threads ()
   ;; Grabbing this lock prevents new threads from
   ;; being spawned, and guarantees that *ALL-THREADS*
@@ -1291,6 +1266,11 @@ on this semaphore, then N of them is woken up."
           (current *current-thread*)
           (joinees nil)
           (main nil))
+      ;; Don't invoke the debugger on errors in cleanup forms in unwind-protect
+      (setf sb!ext:*invoke-debugger-hook*
+            (lambda (c h)
+              (sb!debug::debugger-disabled-hook c h :quit nil)
+              (abort-thread :allow-exit t)))
       (dolist (thread (list-all-threads))
         (cond ((eq thread current))
               ((main-thread-p thread)
@@ -1322,15 +1302,13 @@ on this semaphore, then N of them is woken up."
           (join-thread main :default t :timeout (time-left)))))))
 
 (defun terminate-session ()
-  #!+sb-doc
   "Kill all threads in session except for this one.  Does nothing if current
 thread is not the foreground thread."
   ;; FIXME: threads created in other threads may escape termination
-  (let ((to-kill
-         (with-session-lock (*session*)
-           (and (eq *current-thread*
-                    (car (session-interactive-threads *session*)))
-                (session-threads *session*)))))
+  (let* ((session *session*)
+         (to-kill (with-session-lock (session)
+                    (and (eq *current-thread* (foreground-thread session))
+                         (session-threads session)))))
     ;; do the kill after dropping the mutex; unwind forms in dying
     ;; threads may want to do session things
     (dolist (thread to-kill)
@@ -1342,7 +1320,6 @@ thread is not the foreground thread."
 
 ;;; called from top of invoke-debugger
 (defun debugger-wait-until-foreground-thread (stream)
-  #!+sb-doc
   "Returns T if thread had been running in background, NIL if it was
 interactive."
   (declare (ignore stream))
@@ -1357,48 +1334,53 @@ interactive."
 (defun get-foreground ()
   #!-sb-thread t
   #!+sb-thread
-  (let ((was-foreground t))
+  (let ((session *session*)
+        (was-foreground t))
     (loop
      (/show0 "Looping in GET-FOREGROUND")
-     (with-session-lock (*session*)
-       (let ((int-t (session-interactive-threads *session*)))
-         (when (eq (car int-t) *current-thread*)
-           (unless was-foreground
-             (format *query-io* "Resuming thread ~A~%" *current-thread*))
-           (return-from get-foreground t))
-         (setf was-foreground nil)
-         (unless (member *current-thread* int-t)
-           (setf (cdr (last int-t))
-                 (list *current-thread*)))
-         (condition-wait
-          (session-interactive-threads-queue *session*)
-          (session-lock *session*)))))))
+     (with-session-lock (session)
+       (symbol-macrolet
+           ((interactive-threads (session-interactive-threads session)))
+         (cond
+           ((null interactive-threads)
+            (setf was-foreground nil
+                  interactive-threads (list *current-thread*)))
+           ((not (eq (first interactive-threads) *current-thread*))
+            (setf was-foreground nil)
+            (unless (member *current-thread* interactive-threads)
+              (setf interactive-threads
+                    (append interactive-threads (list *current-thread*))))
+            (condition-wait
+             (session-interactive-threads-queue session)
+             (session-lock session)))
+           (t
+            (unless was-foreground
+              (format *query-io* "Resuming thread ~A~%" *current-thread*))
+            (return-from get-foreground t))))))))
 
 (defun release-foreground (&optional next)
-  #!+sb-doc
   "Background this thread.  If NEXT is supplied, arrange for it to
 have the foreground next."
   #!-sb-thread (declare (ignore next))
   #!-sb-thread nil
   #!+sb-thread
-  (with-session-lock (*session*)
-    (when (rest (session-interactive-threads *session*))
-      (setf (session-interactive-threads *session*)
-            (delete *current-thread* (session-interactive-threads *session*))))
-    (when next
-      (setf (session-interactive-threads *session*)
-            (list* next
-                   (delete next (session-interactive-threads *session*)))))
-    (condition-broadcast (session-interactive-threads-queue *session*))))
+  (let ((session *session*))
+    (with-session-lock (session)
+      (symbol-macrolet
+          ((interactive-threads (session-interactive-threads session)))
+        (setf interactive-threads
+              (delete *current-thread* interactive-threads))
+        (when (and next (thread-alive-p next))
+          (setf interactive-threads
+                (list* next (delete next interactive-threads))))
+        (condition-broadcast (session-interactive-threads-queue session))))))
 
 (defun interactive-threads (&optional (session *session*))
-  #!+sb-doc
   "Return the interactive threads of SESSION defaulting to the current
 session."
   (session-interactive-threads session))
 
 (defun foreground-thread (&optional (session *session*))
-  #!+sb-doc
   "Return the foreground thread of SESSION defaulting to the current
 session."
   (first (interactive-threads session)))
@@ -1436,33 +1418,12 @@ session."
 #!+sb-thread
 (defun initial-thread-function-trampoline
     (thread setup-sem real-function thread-list arguments arg1 arg2 arg3)
-  ;; In time we'll move some of the binding presently done in C here
-  ;; too.
-  ;;
-  ;; KLUDGE: Here we have a magic list of variables that are not
-  ;; thread-safe for one reason or another.  As people report problems
-  ;; with the thread safety of certain variables, (e.g. "*print-case* in
-  ;; multiple threads broken", sbcl-devel 2006-07-14), we add a few more
-  ;; bindings here.  The Right Thing is probably some variant of
-  ;; Allegro's *cl-default-special-bindings*, as that is at least
-  ;; accessible to users to secure their own libraries.
-  ;;   --njf, 2006-07-15
-  ;;
   ;; As it is, this lambda must not cons until we are ready to run
   ;; GC. Be very careful.
-  (let* ((*current-thread* thread)
-         (*restart-clusters* nil)
-         (*handler-clusters* sb!kernel::**initial-handler-clusters**)
-         (*exit-in-process* nil)
-         (sb!impl::*deadline* nil)
-         (sb!impl::*deadline-seconds* nil)
-         (sb!impl::*step-out* nil)
-         ;; internal reader variables
-         (sb!impl::*token-buf-pool* nil)
-         (sb!impl::*ignored-package-locks* :invalid)
-         (sb!impl::*descriptor-handlers* nil)) ; serve-event
-    (declare (inline make-restart)) ;; to allow DX-allocation
-    ;; Binding from C
+  (let ()
+    (setq *current-thread* thread) ; is thread-local already
+    ;; *ALLOC-SIGNAL* is made thread-local by create_thread_struct()
+    ;; so this assigns into TLS, not the global value.
     (setf sb!vm:*alloc-signal* *default-alloc-signal*)
     (setf (thread-os-thread thread) (current-thread-os-thread))
     (with-mutex ((thread-result-lock thread))
@@ -1532,7 +1493,6 @@ session."
   (values))
 
 (defun make-thread (function &key name arguments ephemeral)
-  #!+sb-doc
   "Create a new thread of NAME that runs FUNCTION with the argument
 list designator provided (defaults to no argument). Thread exits when
 the function returns. The return values of FUNCTION are kept around
@@ -1587,7 +1547,6 @@ See also: RETURN-FROM-THREAD, ABORT-THREAD."
     (or thread (error "Could not create a new thread."))))
 
 (defun join-thread (thread &key (default nil defaultp) timeout)
-  #!+sb-doc
   "Suspend current thread until THREAD exits. Return the result values
 of the thread function.
 
@@ -1695,7 +1654,6 @@ subject to change."
         (setf *thruption-pending* t)))))
 
 (defun interrupt-thread (thread function)
-  #!+sb-doc
   "Interrupt THREAD and make it run FUNCTION.
 
 The interrupt is asynchronous, and can occur anywhere with the exception of
@@ -1772,7 +1730,6 @@ Short version: be careful out there."
              (error 'interrupt-thread-error :thread thread))))))
 
 (defun terminate-thread (thread)
-  #!+sb-doc
   "Terminate the thread identified by THREAD, by interrupting it and
 causing it to call SB-EXT:ABORT-THREAD with :ALLOW-EXIT T.
 
@@ -1814,7 +1771,6 @@ assume that unknown code can safely be terminated using TERMINATE-THREAD."
 
 (define-alien-routine "thread_yield" int)
 
-#!+sb-doc
 (setf (fdocumentation 'thread-yield 'function)
       "Yield the processor to other threads.")
 
@@ -1824,6 +1780,12 @@ assume that unknown code can safely be terminated using TERMINATE-THREAD."
 ;;; should probably discuss with a professional psychiatrist first
 #!+sb-thread
 (progn
+
+  (sb!ext:defglobal sb!vm::*free-tls-index* 0)
+  ;; Keep in sync with 'compiler/generic/parms.lisp'
+  #!+ppc ; only PPC uses a separate symbol for the TLS index lock
+  (!defglobal sb!vm::*tls-index-lock* 0)
+
   (defun %thread-sap (thread)
     (let ((thread-sap (alien-sap (extern-alien "all_threads" (* t))))
           (target (thread-os-thread thread)))
@@ -1894,7 +1856,6 @@ assume that unknown code can safely be terminated using TERMINATE-THREAD."
           finally (return seen))))
 
 (defun symbol-value-in-thread (symbol thread &optional (errorp t))
-  #!+sb-doc
   "Return the local value of SYMBOL in THREAD, and a secondary value of T
 on success.
 
