@@ -132,13 +132,51 @@
     (rename-file x path)))
 (compile 'rename-file-a-la-unix)
 
-;;; other miscellaneous tools
-(load "src/cold/read-from-file.lisp")
-(load "src/cold/rename-package-carefully.lisp")
-(load "src/cold/with-stuff.lisp")
+(export '(prepend-genfile-path read-from-file *generated-sources-root*))
+(defvar *generated-sources-root* "")
+
+;;; See remark in COMPILE-STEM about strings vs. The Common Lisp Way
+(defun prepend-genfile-path (namestring)
+  (concatenate 'string
+               ;; if exact match to "output/", or mismatch at the next character
+               (if (member (mismatch "output/" namestring) '(nil 7))
+                   *generated-sources-root*
+                   "")
+               namestring))
+(compile 'prepend-genfile-path) ; seems in vogue to compile everything in this file
+
+;;; Return an expression read from the file named NAMESTRING.
+;;; For user-supplied inputs, protect against more than one expression
+;;; appearing in the file. With trusted inputs we needn't bother.
+(defun read-from-file (namestring)
+  (with-open-file (s (prepend-genfile-path namestring))
+    (let* ((result (read s))
+           (eof-result (cons nil nil)))
+      (when (string= (pathname-name namestring) "build-order")
+        ;; build-order uses both host and target conditionals.
+        ;; Our sanity checks during make-host-1 would complain
+        ;; about #+feature when reading the second expression in the file
+        ;; whenever such feature is also a possible target feature.
+        (return-from read-from-file result))
+      (unless (eq (read s nil eof-result) eof-result)
+        (error "more than one expression in file ~S" namestring))
+      result)))
+(compile 'read-from-file)
 
 ;;; Try to minimize/conceal any non-standardness of the host Common Lisp.
-(load "src/cold/ansify.lisp")
+#-sbcl (load "src/cold/ansify.lisp")
+
+;;;; Do not put SBCL-specific things in 'ansify'. Put them here.
+;;;; And there had better not be a reason that SBCL needs ansification.
+#+sbcl
+(progn
+  (setq *compile-print* nil)
+  (load "src/cold/muffler.lisp")
+  ;; Let's just say we never care to see these.
+  (declaim (sb-ext:muffle-conditions
+            (satisfies unable-to-optimize-note-p)
+            (satisfies optional+key-style-warning-p)
+            sb-ext:code-deletion-note)))
 
 ;;;; special read-macros for building the cold system (and even for
 ;;;; building some of our tools for building the cold system)
@@ -172,13 +210,6 @@
                          #'identity)))
     (funcall customizer default-subfeatures)))
 
-(progn (write-string "target *FEATURES* = ")
-       (write *shebang-features* :pretty nil :length nil)
-       (terpri)
-       (write-string "*shebang-backend-subfeatures* = ")
-       (write *shebang-backend-subfeatures* :pretty nil :length nil)
-       (terpri))
-
 ;;; Call for effect of signaling an error if no target picked.
 (target-platform-name)
 
@@ -318,9 +349,15 @@
       stem)))
 (compile 'stem-remap-target)
 
-;;; Determine the source path for a stem.
+;;; Determine the source path for a stem by remapping from the abstract name
+;;; if it contains "/target/" and appending a ".lisp" suffix.
+;;; Assume that STEM is source-tree-relative unless it starts with "output/"
+;;; in which case it could be elsewhere, if you prefer to keep the sources
+;;; devoid of compilation artifacts. (The production of out-of-tree artifacts
+;;; is not actually implemented in the generic build, however if your build
+;;; system does that by itself, then hooray for you)
 (defun stem-source-path (stem)
-  (concatenate 'string "" (stem-remap-target stem) ".lisp"))
+  (concatenate 'string (prepend-genfile-path (stem-remap-target stem)) ".lisp"))
 (compile 'stem-source-path)
 
 ;;; Determine the object path for a stem/flags/mode combination.
@@ -410,7 +447,7 @@
          (ignore-failure-p (find :ignore-failure-p flags)))
     (declare (type function compile-file))
 
-    (ensure-directories-exist obj :verbose t)
+    (ensure-directories-exist obj :verbose *compile-print*) ; host's value
 
     ;; We're about to set about building a new object file. First, we
     ;; delete any preexisting object file in order to avoid confusing
@@ -505,8 +542,7 @@
 (defun in-host-compilation-mode (fn)
   (declare (type function fn))
   (let ((*features* (cons :sb-xc-host *features*)))
-    (with-additional-nickname ("SB-XC" "SB!XC")
-      (funcall fn))))
+    (funcall fn)))
 (compile 'in-host-compilation-mode)
 
 ;;; Process a file as source code for the cross-compiler, compiling it
@@ -532,12 +568,22 @@
 ;;;; tools to compile SBCL sources to create object files which will
 ;;;; be used to create the target SBCL .core file
 
+(defun lpnify-stem (stem)
+  ;; Don't want genfiles path to sneak in - avoid (STEM-SOURCE-PATH ...) here.
+  (let ((string (stem-remap-target stem)))
+    ;; Distrust that random hosts don't bork up the translation.
+    ;; Simply replace '/' with ';' and be done.
+    (format nil "SYS:~:@(~A~).LISP" (substitute #\; #\/ string))))
+(compile 'lpnify-stem)
+
 ;;; Run the cross-compiler on a file in the source directory tree to
 ;;; produce a corresponding file in the target object directory tree.
 (defun target-compile-stem (stem flags)
   (funcall *in-target-compilation-mode-fn*
            (lambda ()
-             (compile-stem stem flags :target-compile))))
+             (progv (list (intern "*SOURCE-NAMESTRING*" "SB!C"))
+                    (list (lpnify-stem stem))
+               (compile-stem stem flags :target-compile)))))
 (compile 'target-compile-stem)
 
 ;;; (This function is not used by the build process, but is intended
