@@ -953,6 +953,8 @@ One of :WARN, :ERROR or :NONE.")
       (format t " <deleted>"))
 
     (pprint-newline :mandatory)
+    (when (block-start-cleanup block)
+      (format t "cleanup ~s~%" (cleanup-kind (block-start-cleanup block))))
     (awhen (block-info block)
       (format t "start stack: ")
       (print-lvar-stack (ir2-block-start-stack it))
@@ -1033,7 +1035,9 @@ One of :WARN, :ERROR or :NONE.")
       (pprint-newline :mandatory))
     (let ((succ (block-succ block)))
       (format t "successors~{ c~D~}~%"
-              (mapcar (lambda (x) (cont-num (block-start x))) succ))))
+              (mapcar (lambda (x) (cont-num (block-start x))) succ)))
+    (when (block-end-cleanup block)
+      (format t "cleanup ~s~%" (cleanup-kind (block-end-cleanup block)))))
   (values))
 
 ;;; Print the guts of a TN. (logic shared between PRINT-OBJECT (TN T)
