@@ -463,6 +463,47 @@ body, references to a NAME will effectively be replaced with the EXPANSION."
 Return VALUE without evaluating it."
   (reference-constant start next result thing))
 
+;;; We now have a switch to decide whether relative pathnames
+;;; can be stored in fasl files as their source name.
+;;; Regardless of what ANSI says must be bound to *fooNAME* specials,
+;;; the only sane choice for this switch is to use untruenames,
+;;; because compilers should record source filenames _as_given_ for
+;;; later consumption by linkers and debuggers. Attempting to convert
+;;; to a truename fails miserably on symlink forests and
+;;; content-addressable filesystems. Compare to a typical C compiler:
+;;;  $ mkdir -p /tmp/a/b/c/d/e/f/g
+;;;  $ cd /tmp/a/b/c/d
+;;;  $ touch e/f/g/h.i
+;;;  $ cc -g3 -S -o - e/f/g/h.i | grep file
+;;;    .file        "h.i"
+;;;    .file 1 "e/f/g/h.i"
+;;; It has emitted a debug info with just a tail, and a debug info with
+;;; the unaltered pathname, and not a truename in sight.
+;;;
+;;; But in Lisp we merge pathnames with the defaults, because CLHS says that
+;;; "*compile-file-pathname* ... is bound to (pathname (merge-pathnames input-file))."
+;;; and similarly for *load-pathname*.
+;;; It's an implementation detail that we use the same information
+;;; to bind *compile-file-pathname* and to store in the fasl though.
+;;; But CLHS does *NOT* say that that the pathname of a source files as
+;;; represented in its corresponding compiled object is a fully merged name
+;;; - how would it? obtaining the source path isn't a specified thing -
+;;; and that's our loophole to allow the choice.
+;;; Another loophole is that *DEFAULT-PATHNAME-DEFAULTS* can always be #"".
+
+;;; We could invent internal variables, like *COMPILE-FILE-PATHNAME-UNMERGED*
+;;; to hold the right (useful) thing, when the *COMPILE-FILE-PATHNAME*
+;;; holds the un-useful thing.
+;;; Anyway, long story short, merging is all the more wrong when we went
+;;; to heroic efforts to reverse-engineer the original pathname by
+;;; scanning for "src/" as used to be done in LPNIFY-NAMESTRING.
+;;; So name-contexts, at least in self-build, should use untruenameized
+;;; un-merged pathnames. I'm not daring enough to change it for everyone.
+;;; It defaults to what it should, and is changed before saving the image.
+;;;
+(declaim (type (member pathname truename) *name-context-file-path-selector*))
+(defglobal *name-context-file-path-selector* 'pathname)
+
 (defun name-context ()
   ;; Name of the outermost non-NIL BLOCK, or the source namestring
   ;; of the source file.
@@ -479,10 +520,10 @@ Return VALUE without evaluating it."
                                      (neq :catch (cleanup-kind (entry-cleanup (pop b)))))))
                             (lexenv-blocks *lexenv*) :from-end t))
               *source-namestring*
-              (let* ((p (or sb!xc:*compile-file-truename* *load-truename*)))
-                (when p
-                  #+sb-xc-host (lpnify-namestring (namestring p) (pathname-directory p) (pathname-type p))
-                  #-sb-xc-host (namestring p))))))
+              (awhen (case *name-context-file-path-selector*
+                       (pathname (or sb!xc:*compile-file-pathname* *load-pathname*))
+                       (truename (or sb!xc:*compile-file-truename* *load-truename*)))
+                (namestring it)))))
     (when context
       (list :in context))))
 
@@ -586,13 +627,49 @@ be a lambda expression."
              name)
         fallback)))
 
+(def-ir1-translator global-function-preserve-cast
+    ((name original-lvar) start next result)
+  (let* ((cast (lvar-use original-lvar))
+         (value-ctran (make-ctran))
+         (value-lvar (make-lvar))
+         (new-cast (etypecase cast
+                     (function-designator-cast
+                      (make-function-designator-cast
+                       :%type-check  (cast-%type-check cast)
+                       :asserted-type (cast-asserted-type cast)
+                       :type-to-check (cast-type-to-check cast)
+                       :value value-lvar
+                       :derived-type (values-specifier-type '(values function &optional))
+                       :deps (function-designator-cast-deps cast)
+                       :arg-specs (function-designator-cast-arg-specs cast)
+                       :result-specs (function-designator-cast-result-specs cast)
+                       :caller (function-designator-cast-caller cast)
+                       :source-path (cast-source-path cast)))
+                     (cast
+                      (%make-cast
+                       :%type-check (cast-%type-check cast)
+                       :asserted-type (cast-asserted-type cast)
+                       :type-to-check (cast-type-to-check cast)
+                       :value value-lvar
+                       :derived-type (values-specifier-type '(values function &optional))
+                       :source-path (cast-source-path cast))))))
+    (with-fun-name-leaf (leaf name start :global-function t)
+      (reference-leaf start value-ctran value-lvar leaf))
+    (link-node-to-previous-ctran new-cast value-ctran)
+    (setf (lvar-dest value-lvar) new-cast)
+    (use-continuation new-cast next result)))
+
 (defun ensure-lvar-fun-form (lvar lvar-name &optional give-up)
   (aver (and lvar-name (symbolp lvar-name)))
   (if (csubtypep (lvar-type lvar) (specifier-type 'function))
       lvar-name
       (let ((cname (lvar-constant-global-fun-name lvar)))
         (cond (cname
-               `(global-function ,cname))
+               ;; Don't lose type restrictions
+               (if (and (cast-p (lvar-use lvar))
+                        (fun-type-p (cast-asserted-type (lvar-use lvar))))
+                   `(global-function-preserve-cast ,cname ,lvar)
+                   `(global-function ,cname)))
               (give-up
                (give-up-ir1-transform
                 ;; No ~S here because if fallback is shown, it wants no quotes.
@@ -675,6 +752,14 @@ VALUE forms. The variables are bound in parallel after all of the VALUES forms
 have been evaluated."
   (cond ((null bindings)
          (ir1-translate-locally body start next result))
+        ;; This is just to avoid leaking non-standard special forms
+        ;; into macroexpanded code
+        #!-c-stack-is-control-stack
+        ((and (equal bindings '((*alien-stack-pointer* *alien-stack-pointer*))))
+         (ir1-convert start next result
+                      (let ((nsp (gensym "NSP")))
+                        `(let ((,nsp (%primitive current-nsp)))
+                           (restoring-nsp ,nsp ,@body)))))
         ((listp bindings)
          (multiple-value-bind (forms decls) (parse-body body nil)
            (multiple-value-bind (vars values) (extract-let-vars bindings 'let)
@@ -874,9 +959,11 @@ other."
     (cond ((or (eq type *wild-type*)
                (eq type *universal-type*)
                (and (leaf-p value)
+                    (not (fun-designator-type-p type))
                     (values-subtypep (make-single-value-type (leaf-type value))
                                      type))
-               (and (sb!xc:constantp value)
+               (and (not (fun-designator-type-p type))
+                    (sb!xc:constantp value)
                     (or (not (values-type-p type))
                         (values-type-may-be-single-value-p type))
                     (ctypep (constant-form-value value)
@@ -963,35 +1050,11 @@ care."
       (setf (lvar-dest index-lvar) cast)
       (use-continuation cast next result))))
 
-;;; Checks at compile time that the function designator can accept
-;;; ARG-COUNT arguments. Can't exactly use THE since
-;;; (function (t t)) doesn't match functions with &optional or &rest
-;;; doesn't include symbols
-(def-ir1-translator callable-cast ((arg-count caller value) start next result)
-  (let ((value-ctran (make-ctran))
-        (value-lvar (make-lvar))
-        (policy (lexenv-policy *lexenv*)))
-    (ir1-convert start value-ctran value-lvar value)
-    (let* ((type (specifier-type 'callable))
-           (cast (make-function-designator-cast :asserted-type type
-                                                :value value-lvar
-                                                :type-to-check (maybe-weaken-check type policy)
-                                                :derived-type (coerce-to-values type)
-                                                :caller caller
-                                                :arg-count arg-count)))
-      (link-node-to-previous-ctran cast value-ctran)
-      (setf (lvar-dest value-lvar) cast)
-      (use-continuation cast next result))))
-
 #-sb-xc-host
 (setf (info :function :macro-function 'truly-the)
       (lambda (whole env)
         (declare (ignore env))
         `(the ,@(cdr whole)))
-      (info :function :macro-function 'callable-cast)
-      (lambda (whole env)
-        (declare (ignore env))
-        `(the callable ,@(cdddr whole)))
       (info :function :macro-function 'the*)
       (lambda (whole env)
         (declare (ignore env))
@@ -1147,6 +1210,28 @@ the thrown values will be returned."
          :catch (%catch (%escape-fun ,exit-block) ,tag)
          ,@body)))))
 
+;;; Since NSP is restored on unwind we only need to protect against
+;;; local transfers of control, basically the same as special
+;;; bindings.
+;;; Needs to be wrapped in a LET (let ((nsp (current-nsp))) (restoring-nsp nsp body))
+;;; The LET is needed because the cleanup can be emitted multiple
+;;; times, but there's no reference to NSP before EMIT-CLEANUPS.
+;;; Passing NSP to the dummy %CLEANUP-FUN keeps it alive.
+#!-c-stack-is-control-stack
+(def-ir1-translator restoring-nsp
+    ((nsp &body body) start next result)
+  (let ((cleanup (make-cleanup :kind :restore-nsp))
+        (nsp-ctran (make-ctran))
+        (cleanup-ctran (make-ctran)))
+    (ir1-convert start nsp-ctran nil nsp)
+    (setf (cleanup-mess-up cleanup) (ctran-use nsp-ctran))
+    (let ((*lexenv* (make-lexenv :cleanup cleanup)))
+      ;; KLUDGE: reference NSP twice so that the LET doesn't get
+      ;; deleted before EMIT-CLEANUPS
+      (ir1-convert nsp-ctran cleanup-ctran nil `(%cleanup-point ,nsp ,nsp))
+      (ir1-convert-progn-body cleanup-ctran next result
+                              body))))
+
 (def-ir1-translator unwind-protect
     ((protected &body cleanup) start next result)
   "UNWIND-PROTECT protected cleanup*
@@ -1223,10 +1308,21 @@ values from the first VALUES-FORM making up the first argument, etc."
 
 Evaluate VALUES-FORM and then the FORMS, but return all the values of
 VALUES-FORM."
-  (let ((dummy (make-ctran)))
-    (ctran-starts-block dummy)
-    (ir1-convert start dummy result values-form)
-    (ir1-convert-progn-body dummy next nil forms)))
+  (let* ((value-ctran (make-ctran))
+         (forms-ctran (make-ctran))
+         (value-lvar (make-lvar))
+         ;; This is to avoid writing in the RESULT LVAR before the
+         ;; body is executed, because the body may overwrite it.
+         ;; See MAY-DELETE-VESTIGIAL-EXIT.
+         (cast (make-vestigial-exit-cast
+                :value value-lvar)))
+    (ctran-starts-block value-ctran)
+    (ir1-convert start value-ctran value-lvar values-form)
+    (ir1-convert-progn-body value-ctran forms-ctran nil forms)
+    (link-node-to-previous-ctran cast forms-ctran)
+    (setf (lvar-dest value-lvar) cast)
+    (use-continuation cast next result)))
+
 
 ;;;; interface to defining macros
 
