@@ -84,11 +84,12 @@
   (let ((block (tn-local tn))
         (num (tn-local-number tn)))
     (add-global-conflict
-     (if (zerop (sbit (ir2-block-written block) num))
-         :read-only
-         (if (zerop (sbit (ir2-block-live-out block) num))
-             :write
-             :read))
+     (cond ((zerop (sbit (ir2-block-written block) num))
+            :read-only)
+           ((zerop (sbit (ir2-block-live-out block) num))
+            :write)
+           (t
+            :read))
      tn block num))
   (values))
 
@@ -646,7 +647,8 @@
         (ecase (tn-kind tn)
           ((:normal :debug-environment)
            (setf (sbit live (tn-local-number tn)) 0))
-          (:environment :component))))
+          (:environment :component)
+          (:unused))))
     live))
 
 ;;; This is used to determine whether a :DEBUG-ENVIRONMENT TN should
@@ -676,7 +678,8 @@
                (not (eq (ir2-block-block next) 1block))
                (or (null succ)
                    (eq (first succ)
-                       (component-tail (block-component 1block)))))
+                       (component-tail (block-component 1block)))
+                   (block-tail-local-call-fun 1block)))
       (do ((conf (ir2-block-global-tns block)
                  (global-conflicts-next-blockwise conf)))
           ((null conf))
