@@ -89,7 +89,8 @@
 (defun merge-tail-sets (call &optional (new-fun (combination-lambda call)))
   (declare (type basic-combination call) (type clambda new-fun))
   (let ((return (node-dest call)))
-    (when (return-p return)
+    (when (and (return-p return)
+               (not (memq (functional-kind new-fun) '(:deleted :zombie))))
       (let ((call-set (lambda-tail-set (node-home-lambda call)))
             (fun-set (lambda-tail-set new-fun)))
         (unless (eq call-set fun-set)
@@ -333,7 +334,9 @@
                       (eq (lvar-uses lvar) ref))
 
                  (convert-call-if-possible ref dest)
-
+                 ;; It might have been deleted by CONVERT-CALL-IF-POSSIBLE
+                 (when (eq (functional-kind fun) :deleted)
+                   (return-from locall-analyze-fun-1))
                  (unless (eq (basic-combination-kind dest) :local)
                    (reference-entry-point ref)
                    (setq local-p nil)))
@@ -1043,8 +1046,9 @@
                      (lvar (node-lvar call)))
                  (unlink-blocks block (first (block-succ block)))
                  (link-blocks block next-block)
-                 (aver (not (node-lvar this-call)))
-                 (add-lvar-use this-call lvar)))
+                 (if (eq (node-derived-type this-call) *empty-type*)
+                     (maybe-terminate-block this-call nil)
+                     (add-lvar-use this-call lvar))))
               (:deleted)
               ;; The called function might be an assignment in the
               ;; case where we are currently converting that function.
@@ -1087,9 +1091,15 @@
          (call-return (lambda-return call-fun)))
     (when (and call-return
                (block-delete-p (node-block call-return)))
+      (flush-dest (return-result call-return))
       (delete-return call-return)
-      (unlink-node call-return)
-      (setq call-return nil))
+      ;; A new return will be put into that lambda, don't want
+      ;; DELETE-RETURN called by DELETE-BLOCK to delete the new return
+      ;; from the lambda.
+      ;; (Previously, UNLINK-NODE was called on the return, but it
+      ;; doesn't work well on deleted blocks)
+      (setf (return-lambda call-return) nil
+            call-return nil))
     (cond ((not return))
           ((or next-block call-return)
            (unless (block-delete-p (node-block return))
@@ -1102,7 +1112,7 @@
            (setf (lambda-return call-fun) return)
            (setf (return-lambda return) call-fun)
            (setf (lambda-return fun) nil))))
-  (%delete-lvar-use call) ; LET call does not have value semantics
+  (delete-lvar-use call) ; LET call does not have value semantics
   (values))
 
 ;;; Actually do LET conversion. We call subfunctions to do most of the
@@ -1358,7 +1368,8 @@
       (when (and (dolist (ref (leaf-refs clambda) t)
                    (let ((dest (node-dest ref)))
                      (when (or (not dest)
-                               (block-delete-p (node-block dest)))
+                               (node-to-be-deleted-p ref)
+                               (node-to-be-deleted-p dest))
                        (return nil))
                      (let ((home (node-home-lambda ref)))
                        (unless (eq home clambda)
