@@ -209,7 +209,7 @@
               (move ,temp-tn ,size)))
 
        #!-sb-thread
-       (inst lr ,flag-tn (make-fixup "boxed_region" :foreign))
+       (inst lr ,flag-tn (make-fixup "gc_alloc_region" :foreign))
        #!-sb-thread
        (inst lwz ,result-tn ,flag-tn 0)
        #!+sb-thread
@@ -218,7 +218,7 @@
 
        ;; we can optimize this to only use one fixup here, once we get
        ;; it working
-       ;; (inst lr ,flag-tn (make-fixup "boxed_region" :foreign 4))
+       ;; (inst lr ,flag-tn (make-fixup "gc_alloc_region" :foreign 4))
        ;; (inst lwz ,flag-tn ,flag-tn 0)
        #!-sb-thread
        (inst lwz ,flag-tn ,flag-tn 4)
@@ -247,7 +247,7 @@
          ;; #!-sb-thread three #!+sb-thread one machine instruction.
          ;; The lr of a fixup counts as two instructions.
          #!-sb-thread
-         (inst lr ,flag-tn (make-fixup "boxed_region" :foreign))
+         (inst lr ,flag-tn (make-fixup "gc_alloc_region" :foreign))
          #!-sb-thread
          (inst stw ,result-tn ,flag-tn 0)
          #!+sb-thread
@@ -309,13 +309,7 @@
     (encode-internal-error-args values)
     (emit-alignment word-shift)))
 
-(defun error-call (vop error-code &rest values)
-  #!+sb-doc
-  "Cause an error.  ERROR-CODE is the error to cause."
-  (emit-error-break vop error-trap (error-number-or-lose error-code) values))
-
 (defun generate-error-code (vop error-code &rest values)
-  #!+sb-doc
   "Generate-Error-Code Error-code Value*
   Emit code for an error with the specified Error-Code and context Values."
   (assemble (*elsewhere*)
@@ -363,18 +357,3 @@
 #!+sb-safepoint
 (defun emit-safepoint ()
   (inst lwz zero-tn null-tn (- (+ 4096 4 other-pointer-lowtag))))
-
-(sb!xc:defmacro with-pinned-objects ((&rest objects) &body body)
-  "Arrange with the garbage collector that the pages occupied by
-OBJECTS will not be moved in memory for the duration of BODY.
-Useful for e.g. foreign calls where another thread may trigger
-garbage collection.  This is currently implemented by disabling GC"
-  #!-gencgc
-  (declare (ignore objects))            ; should we eval these for side-effect?
-  #!-gencgc
-  `(without-gcing
-    ,@body)
-  #!+gencgc
-  `(let ((*pinned-objects* (list* ,@objects *pinned-objects*)))
-     (declare (truly-dynamic-extent *pinned-objects*))
-     ,@body))
