@@ -43,11 +43,6 @@
   (declare (type list insts))
   (sort insts #'> :key #'specializer-rank))
 
-(defun specialization-error (insts)
-  (bug
-   "~@<Instructions either aren't related or conflict in some way: ~4I~_~S~:>"
-   insts))
-
 ;;; Given a list of instructions INSTS, Sees if one of these instructions is a
 ;;; more general form of all the others, in which case they are put into its
 ;;; specializers list, and it is returned. Otherwise an error is signaled.
@@ -62,7 +57,8 @@
           (return)                      ; exit the inner loop
           )))
     (cond ((null masters)
-           (specialization-error insts))
+           (bug "~@<Instructions either aren't related or conflict in some way: ~4I~_~S~:>"
+                insts))
           ((cdr masters)
            (error "multiple specializing masters: ~S" masters))
           (t
@@ -225,10 +221,10 @@
 ;;; Function header layout (dual word aligned):
 ;;;     header-word
 ;;;     self pointer
-;;;     next pointer (next function header)
 ;;;     name
 ;;;     arglist
 ;;;     type
+;;;     info
 ;;;
 ;;; LRA layout (dual word aligned):
 ;;;     header-word
@@ -373,7 +369,7 @@
   (declare (type disassem-state dstate)
            (type segment segment))
   (setf (dstate-segment dstate) segment)
-  (setf (dstate-inst-properties dstate) nil)
+  (setf (dstate-inst-properties dstate) 0)
   (setf (dstate-cur-offs-hooks dstate)
         (stable-sort (nreverse (copy-list (seg-hooks segment)))
                      (lambda (oh1 oh2)
@@ -659,7 +655,7 @@
                                  (dstate-filtered-arg-pool-free dstate))
                   (setq arg saved-next))))
         (setf (dstate-filtered-arg-pool-in-use dstate) nil)
-        (setf (dstate-inst-properties dstate) nil)))))
+        (setf (dstate-inst-properties dstate) 0)))))
 
 
 (defun collect-labelish-operands (args cache)
@@ -802,9 +798,10 @@
               (collect-inst-variants
                (logically-readonlyize (string-upcase symbol))
                package it cache))))
-    (apply 'format t
-           "~&Disassembler: ~D printers, ~D prefilters, ~D labelers~%"
-           (mapcar (lambda (x) (length (cdr x))) cache))))
+    (unless (sb!impl::!c-runtime-noinform-p)
+      (apply 'format t
+             "~&Disassembler: ~D printers, ~D prefilters, ~D labelers~%"
+             (mapcar (lambda (x) (length (cdr x))) cache)))))
 
 ;;; Get the instruction-space, creating it if necessary.
 (defun get-inst-space (&key (package sb!assem::*backend-instruction-set-package*)
@@ -1001,6 +998,15 @@
                   :alignment alignment
                   :byte-order sb!c:*backend-byte-order*)))
 
+;;; Logically or MASK into the set of instruction properties in DSTATE.
+(defun dstate-setprop (dstate mask)
+  (setf (dstate-inst-properties dstate) (logior mask (dstate-inst-properties dstate))))
+
+;;; Return non-NIL if any bit in MASK
+;;; is in the set of instruction properties in DSTATE.
+(defun dstate-getprop (dstate mask)
+  (logtest mask (dstate-inst-properties dstate)))
+
 (defun add-fun-header-hooks (segment)
   (declare (type segment segment))
   (dotimes (i (or (awhen (seg-code segment) (code-n-entries it)) 0))
@@ -1025,7 +1031,7 @@
 ;;; A SAP-MAKER is a no-argument function that returns a SAP.
 
 ;; FIXME: Are the objects we are taking saps for always pinned?
-#!-sb-fluid (declaim (inline sap-maker))
+(declaim (inline sap-maker))
 (defun sap-maker (function input offset)
   (declare (optimize (speed 3))
            (muffle-conditions compiler-note)
