@@ -28,6 +28,14 @@
 (deftype sc-vector () `(simple-vector ,sc-number-limit))
 (deftype sc-bit-vector () `(simple-bit-vector ,sc-number-limit))
 
+(deftype sc-locations ()
+  '(simple-array sc-offset 1))
+(declaim (inline make-sc-locations))
+(defun make-sc-locations (locations)
+  (make-array (length locations)
+              :element-type 'sc-offset
+              :initial-contents locations))
+
 ;;; the different policies we can use to determine the coding strategy
 (deftype ltn-policy ()
   '(member :safe :small :fast :fast-safe))
@@ -692,10 +700,7 @@
 
 ;;; The SB structure represents the global information associated with
 ;;; a storage base.
-(defstruct (sb (:copier nil))
-  ;; for indirecting access of read/write slots of this SB into an
-  ;; identically indexed SB object local to the particular compilation
-  (%index 0 :type (mod 8) :read-only t) ; arbitrary limit
+(defstruct (storage-base (:copier nil) (:conc-name sb-))
   ;; name, for printing and reference
   (name nil :type symbol :read-only t)
   ;; the kind of storage base (which determines the packing
@@ -705,18 +710,22 @@
   ;; size. If unbounded, this is the size that the SB is initially
   ;; allocated at.
   (size 0 :type index :read-only t))
-(defprinter (sb)
-  name)
 
 ;;; A FINITE-SB holds information needed by the packing algorithm for
 ;;; finite SBs.
-(defstruct (finite-sb (:include sb) (:copier nil) (:conc-name fsb-))
+(defstruct (finite-sb-template (:include storage-base) (:copier nil)
+                               (:predicate nil) (:conc-name finite-sb-))
+  ;; for indirecting access of read/write slots of this SB into
+  ;; a proxy object specific to one compiler invocation.
+  (index 0 :type (mod #+sb-xc-host 8 ; arbitrary limit
+                      #-sb-xc-host #.(length *finite-sbs*)) :read-only t)
   ;; the minimum number of location by which to grow this SB
   ;; if it is :unbounded
   (size-increment 1 :type index :read-only t)
   ;; current-size must always be a multiple of this. It is assumed
   ;; to be a power of two.
-  (size-alignment 1 :type index :read-only t)
+  (size-alignment 1 :type index :read-only t))
+(defstruct (finite-sb (:copier nil) (:predicate nil) (:conc-name fsb-))
   ;; the number of locations currently allocated in this SB
   (current-size 0 :type index)
   ;; the last location packed in, used by pack to scatter TNs to
@@ -743,28 +752,20 @@
   ;; starts. Less then the length of those vectors when not all of the
   ;; length was used on the previously packed component.
   (last-block-count 0 :type index))
-(proclaim '(freeze-type sb finite-sb))
-
-;;; During make-host-1 the length of *FINITE-SBS* is unknown.
-#+sb-xc-host
-(defmacro finite-sb-index (sb) `(fsb-%index ,sb))
-;;; The target compiler can be more efficient by eliding the bounds check.
-#-sb-xc-host
-(defmacro finite-sb-index (sb)
-  ;; Using #. instead of ",(length *backend-sbs*)" because the cross-compiler
-  ;; has not seen *BACKEND-SBS* yet, but the host compiler has!
-  `(truly-the (mod #.(length *backend-sbs*)) (fsb-%index ,sb)))
+(declaim (freeze-type storage-base finite-sb-template finite-sb))
 
 ;;; Give this a toplevel value so that it can be declaimed ALWAYS-BOUND.
 ;;; The compiler will never look at the toplevel value though.
-(defvar *finite-sbs* #-sb-xc-host (make-array #.(length *backend-sbs*)))
-#-sb-xc-host (declaim (type (simple-vector #.(length *backend-sbs*))
-                            *finite-sbs*)
-                      (always-bound *finite-sbs*))
-;;; These two are not SETFable
-(declaim (inline finite-sb-size-increment finite-sb-size-alignment))
-(defun finite-sb-size-increment (sb) (fsb-size-increment sb))
-(defun finite-sb-size-alignment (sb) (fsb-size-alignment sb))
+(defvar *finite-sbs*
+  #-sb-xc-host
+  (make-array #.(count :non-packed *backend-sbs* :key #'sb-kind :test #'neq)))
+#-sb-xc-host
+(progn
+  (declaim (type (simple-vector #.(length *finite-sbs*)) *finite-sbs*)
+           (always-bound *finite-sbs*))
+  (eval-when (:compile-toplevel :load-toplevel :execute)
+    (setf (info :variable :wired-tls '*finite-sbs*) :always-thread-local)))
+
 ;;; All these are SETFable
 (defmacro finite-sb-current-size (sb)
   `(fsb-current-size (svref *finite-sbs* (finite-sb-index ,sb))))
@@ -789,11 +790,11 @@
   ;; the number used to index SC cost vectors
   (number 0 :type sc-number)
   ;; the storage base that this SC allocates storage from
-  (sb nil :type (or sb null))
+  (sb nil :type (or storage-base null))
   ;; the size of elements in this SC, in units of locations in the SB
   (element-size 0 :type index)
-  ;; if our SB is finite, a list of the locations in this SC
-  (locations nil :type list)
+  ;; if our SB is finite, a vector of the locations in this SC
+  (locations (missing-arg) :type sc-locations :read-only t)
   ;; a list of the alternate (save) SCs for this SC
   (alternate-scs nil :type list)
   ;; a list of the constant SCs that can me moved into this SC
@@ -846,11 +847,11 @@
   ;; alignment restriction. The offset must be an even multiple of this.
   ;; this must be a power of two.
   (alignment 1 :type (and index (integer 1)))
-  ;; a list of locations that we avoid packing in during normal
+  ;; a vector of locations that we avoid packing in during normal
   ;; register allocation to ensure that these locations will be free
   ;; for operand loading. This prevents load-TN packing from thrashing
   ;; by spilling a lot.
-  (reserve-locations nil :type list))
+  (reserve-locations (missing-arg) :type sc-locations :read-only t))
 (defprinter (sc)
   name)
 
