@@ -38,8 +38,7 @@
                              temp))))
                      (storew reg ,list ,slot list-pointer-lowtag))))
              (let ((cons-cells (if star (1- num) num))
-                   (stack-allocate-p (awhen (sb!c::node-lvar node)
-                                       (sb!c::lvar-dynamic-extent it))))
+                   (stack-allocate-p (node-stack-allocate-p node)))
                (maybe-pseudo-atomic stack-allocate-p
                 (allocation res (* (pad-data-block cons-size) cons-cells) node
                             stack-allocate-p list-pointer-lowtag)
@@ -94,6 +93,11 @@
                      :disp (- (* slot n-word-bytes) lowtag))
             word)))
 
+;; from 'llvm/projects/compiler-rt/lib/msan/msan.h':
+;;  "#define MEM_TO_SHADOW(mem) (((uptr)(mem)) ^ 0x500000000000ULL)"
+#!+linux ; shadow space differs by OS
+(defconstant msan-mem-to-shadow-xor-const #x500000000000)
+
 ;;; ALLOCATE-VECTOR
 (macrolet ((calc-size-in-bytes (n-words result-tn)
              `(cond ((sc-is ,n-words immediate)
@@ -119,6 +123,8 @@
     (:args (type :scs (unsigned-reg immediate))
            (length :scs (any-reg immediate))
            (words :scs (any-reg immediate)))
+    ;; Result is live from the beginning, like a temp, because we use it as such
+    ;; in 'calc-size-in-bytes'
     (:results (result :scs (descriptor-reg) :from :load))
     (:arg-types positive-fixnum positive-fixnum positive-fixnum)
     (:policy :fast-safe)
@@ -131,26 +137,56 @@
          (put-header result type length t)))))
 
   (define-vop (allocate-vector-on-stack)
-    (:args (type :scs (unsigned-reg immediate) :to :save)
-           (length :scs (any-reg immediate) :to :eval :target rax)
-           (words :scs (any-reg immediate) :target rcx))
-    (:temporary (:sc any-reg :offset ecx-offset :from (:argument 2)) rcx)
-    (:temporary (:sc any-reg :offset eax-offset :from :eval) rax)
-    (:temporary (:sc any-reg :offset edi-offset) rdi)
-    (:temporary (:sc complex-double-reg) zero)
+    (:args (type :scs (unsigned-reg immediate))
+           (length :scs (any-reg immediate))
+           (words :scs (any-reg immediate)))
     (:results (result :scs (descriptor-reg) :from :load))
+    (:temporary (:sc any-reg :offset ecx-offset :from :eval) rcx)
+    (:temporary (:sc any-reg :offset eax-offset :from :eval) rax)
+    (:temporary (:sc any-reg :offset edi-offset :from :eval) rdi)
+    (:temporary (:sc complex-double-reg) zero)
     (:arg-types positive-fixnum positive-fixnum positive-fixnum)
     (:translate allocate-vector)
     (:policy :fast-safe)
     (:node-var node)
     (:generator 100
-      (let ((size (calc-size-in-bytes words result)))
+      (let ((size (calc-size-in-bytes words result))
+            (rax-zeroed))
         (allocation result size node t other-pointer-lowtag)
         (put-header result type length nil)
         ;; FIXME: It would be good to check for stack overflow here.
         ;; It would also be good to skip zero-fill of specialized vectors
         ;; perhaps in a policy-dependent way. At worst you'd see random
         ;; bits, and CLHS says consequences are undefined.
+        (when sb!c::*msan-compatible-stack-unpoison*
+          ;; Unpoison all DX vectors regardless of widetag.
+          ;; Mark the header and length as valid, not just the payload.
+          #!+linux ; unimplemented for others
+          (let ((words-savep
+                 ;; 'words' might be co-located with any of the temps
+                 (or (location= words rdi) (location= words rcx) (location= words rax)))
+                (rax rax))
+            (setq rax-zeroed (not (location= words rax)))
+            (when words-savep ; use 'result' to save 'words'
+              (inst mov result words))
+            (cond ((sc-is words immediate)
+                   (inst mov rcx (+ (tn-value words) vector-data-offset)))
+                  (t
+                   (inst lea rcx
+                         (make-ea :qword :base words
+                                  :disp (ash vector-data-offset n-fixnum-tag-bits)))
+                   (if (= n-fixnum-tag-bits 1)
+                       (setq rax (reg-in-size rax :dword)) ; don't bother shifting rcx
+                       (inst shr rcx n-fixnum-tag-bits))))
+            (inst mov rdi msan-mem-to-shadow-xor-const)
+            (inst xor rdi rsp-tn) ; compute shadow address
+            (zeroize rax)
+            (inst rep)
+            (inst stos rax)
+            (when words-savep
+              (inst mov words result) ; restore 'words'
+              (inst lea result ; recompute the tagged pointer
+                    (make-ea :byte :base rsp-tn :disp other-pointer-lowtag)))))
         (let ((data-addr
                 (make-ea :qword :base result
                                 :disp (- (* vector-data-offset n-word-bytes)
@@ -161,8 +197,7 @@
                      (cond ((> n 8)
                             (inst mov rcx (tn-value words)))
                            ((= n 1)
-                            (zeroize rax)
-                            (inst mov data-addr rax)
+                            (inst mov data-addr 0)
                             (return-from zero-fill))
                            (t
                             (multiple-value-bind (double single) (truncate n 2)
@@ -178,8 +213,7 @@
                    (move rcx words)
                    (inst shr rcx n-fixnum-tag-bits)))
             (inst lea rdi data-addr)
-            (inst cld)
-            (zeroize rax)
+            (unless rax-zeroed (zeroize rax))
             (inst rep)
             (inst stos rax)))))))
 
@@ -283,19 +317,28 @@
               result fdefn-raw-addr-slot other-pointer-lowtag))))
 
 (define-vop (make-closure)
-  (:args (function :to :save :scs (descriptor-reg)))
-  (:info length stack-allocate-p)
+  ; (:args (function :to :save :scs (descriptor-reg)))
+  (:info label length stack-allocate-p)
   (:temporary (:sc any-reg) temp)
   (:results (result :scs (descriptor-reg)))
   (:node-var node)
   (:generator 10
    (maybe-pseudo-atomic stack-allocate-p
-    (let ((size (+ length closure-info-offset)))
+    (let* ((size (+ length closure-info-offset))
+           (header (logior (ash (1- size) n-widetag-bits) closure-widetag)))
       (allocation result (pad-data-block size) node stack-allocate-p
                   fun-pointer-lowtag)
-      (storew* (logior (ash (1- size) n-widetag-bits) closure-header-widetag)
+      (storew* #!-immobile-space header ; write the widetag and size
+               #!+immobile-space        ; ... plus the layout pointer
+               (progn (inst mov temp header)
+                      (inst or temp (static-symbol-value-ea 'function-layout))
+                      temp)
                result 0 fun-pointer-lowtag (not stack-allocate-p)))
-    (loadw temp function closure-fun-slot fun-pointer-lowtag)
+    ;; These two instructions are within the scope of PSEUDO-ATOMIC.
+    ;; This is due to scav_closure() assuming that it can always subtract
+    ;; FUN_RAW_ADDR_OFFSET from closure->fun to obtain a Lisp object,
+    ;; without any precheck for whether that word is currently 0.
+    (inst lea temp (make-fixup nil :closure label))
     (storew temp result closure-fun-slot fun-pointer-lowtag))))
 
 ;;; The compiler likes to be able to directly make value cells.
@@ -306,7 +349,7 @@
   (:node-var node)
   (:generator 10
     (with-fixed-allocation
-        (result value-cell-header-widetag value-cell-size node stack-allocate-p)
+        (result value-cell-widetag value-cell-size node stack-allocate-p)
       (storew value result value-cell-value-slot other-pointer-lowtag))))
 
 ;;;; automatic allocators for primitive objects
@@ -321,20 +364,33 @@
   (:args)
   (:results (result :scs (any-reg)))
   (:generator 1
+    ;; TODO: if in immobile space, use LEA to make position-independent
     (inst mov result (make-fixup 'funcallable-instance-tramp :assembly-routine))))
 
 (define-vop (fixed-alloc)
   (:args)
   (:info name words type lowtag stack-allocate-p)
-  (:ignore name)
   (:results (result :scs (descriptor-reg)))
   (:node-var node)
   (:generator 50
+    (progn name) ; possibly not used
     (maybe-pseudo-atomic stack-allocate-p
      (allocation result (pad-data-block words) node stack-allocate-p lowtag)
      (when type
-       (storew* (logior (ash (1- words) n-widetag-bits) type)
-                result 0 lowtag (not stack-allocate-p))))))
+       (let* ((widetag (if (typep type 'layout) instance-widetag type))
+              (header (logior (ash (1- words) n-widetag-bits) widetag)))
+         (if (or #!+compact-instance-header
+                 (and (eq name '%make-structure-instance) stack-allocate-p))
+             ;; Write a :DWORD, not a :QWORD, because the high half will be
+             ;; filled in when the layout is stored. Can't use STOREW* though,
+             ;; because it tries to store as few bytes as possible,
+             ;; where this instruction must write exactly 4 bytes.
+             (inst mov (make-ea :dword :base result :disp (- lowtag)) header)
+             (storew* header result 0 lowtag (not stack-allocate-p)))
+         (unless (eq type widetag) ; TYPE is actually a LAYOUT
+           (inst mov (make-ea :dword :base result :disp (+ 4 (- lowtag)))
+                 ;; XXX: should layout fixups use a name, not a layout object?
+                 (make-fixup type :layout))))))))
 
 (define-vop (var-alloc)
   (:args (extra :scs (any-reg)))
@@ -387,9 +443,13 @@
   ;; These VOPs are each used in one place only, and deliberately not
   ;; specified as transforming the function after which they are named.
   (def alloc-immobile-layout "alloc_layout" ; MAKE-LAYOUT
-       ((descriptor-reg) (descriptor-reg)))
+       ((descriptor-reg)))
   (def alloc-immobile-symbol "alloc_sym"    ; MAKE-SYMBOL
-       ((descriptor-reg) (any-reg)))
+       ((descriptor-reg)))
   (def alloc-immobile-fdefn  "alloc_fdefn"  ; MAKE-FDEFN
        ((descriptor-reg)))
+  #!+immobile-code (def alloc-fun-tramp "alloc_fun_tramp" ((descriptor-reg)))
+  #!+(and immobile-code compact-instance-header)
+  (def alloc-generic-function  "alloc_generic_function"
+       ((descriptor-reg)))
   )
