@@ -251,21 +251,22 @@
         (when check-boundp
           (assemble ()
             (inst cmp (reg-in-size value :dword) unbound-marker-widetag)
-            (let ((*location-context* (make-restart-location RETRY value)))
-              ;; For IMMEDIATE sc we need to load the symbol. It would be better
-              ;; to encode it into the break info, but:
-              ;;  * there are some comments scattered around indicating
-              ;;    that it doesn't work to encode immediate values.
-              ;;  * I don't feel like figuring out whether that's true
-              ;; Anyway, this keeps the extra MOV instruction out of
-              ;; the normal execution path.
+            (let* ((immediatep (sc-is symbol immediate))
+                   (staticp (and immediatep (static-symbol-p known-symbol)))
+                   (*location-context* (make-restart-location RETRY value)))
+              ;; IMMEDIATE sc symbols are not in a register (they are accessed
+              ;; via absolute address), nor are they present in the code header.
+              ;; So emit a MOV just before the INT opcode for such symbols,
+              ;; out of the normal execution path. Most static symbols are
+              ;; DEFCONSTANTs or DEFGLOBALs, so this case is infrequent.
               (inst jmp :e (generate-error-code+
-                            (if (sc-is symbol immediate)
+                            (if staticp
                                 (lambda ()
-                                  (inst mov symbol-reg
-                                        (make-fixup known-symbol :immobile-object))))
+                                  (load-immediate vop symbol symbol-reg)))
                             vop 'unbound-symbol-error
-                            symbol-reg)))
+                            (if (and immediatep (not staticp))
+                                symbol
+                                symbol-reg))))
             RETRY))))
 
   ;; With Symbol-Value, we check that the value isn't the trap object. So
@@ -477,23 +478,21 @@
 
 #!+sb-thread
 (define-vop (unbind)
-  (:temporary (:sc unsigned-reg) temp bsp tls-index)
+  (:temporary (:sc unsigned-reg) temp bsp)
   (:temporary (:sc complex-double-reg) zero)
-  (:info n)
+  (:info symbols)
   (:generator 0
     (load-binding-stack-pointer bsp)
     (inst xorpd zero zero)
-    (loop repeat n
+    (loop for symbol in symbols
+          for tls-index = (load-time-tls-offset symbol)
+          for tls-cell = (make-ea :qword :base thread-base-tn :disp tls-index)
           do
           (inst sub bsp (* binding-size n-word-bytes))
-          ;; Load TLS-INDEX of the SYMBOL from stack
-          (inst mov (reg-in-size tls-index :dword)
-                (make-ea :dword :base bsp :disp (* binding-symbol-slot n-word-bytes)))
 
           ;; Load VALUE from stack, then restore it to the TLS area.
           (loadw temp bsp binding-value-slot)
-          (inst mov (make-ea :qword :base thread-base-tn :index tls-index)
-                temp)
+          (inst mov tls-cell temp)
           ;; Zero out the stack.
           (inst movapd (make-ea :qword :base bsp) zero))
     (store-binding-stack-pointer bsp)))
@@ -564,11 +563,21 @@
   funcallable-instance-info-offset fun-pointer-lowtag
   (descriptor-reg any-reg) * %funcallable-instance-info)
 
-(define-vop (closure-ref slot-ref)
-  (:variant closure-info-offset fun-pointer-lowtag))
+(define-vop (closure-ref)
+  (:args (object :scs (descriptor-reg)))
+  (:results (value :scs (descriptor-reg any-reg)))
+  (:info offset)
+  (:generator 4
+    (loadw value object (+ closure-info-offset offset) fun-pointer-lowtag)))
 
-(define-vop (closure-init slot-set)
-  (:variant closure-info-offset fun-pointer-lowtag))
+(define-vop (closure-init)
+  (:args (object :scs (descriptor-reg))
+         (value :scs (descriptor-reg any-reg immediate)))
+  (:temporary (:sc unsigned-reg) temp)
+  (:info offset)
+  (:generator 4
+     (generate-set-slot object value temp
+                        (+ closure-info-offset offset) fun-pointer-lowtag)))
 
 (define-vop (closure-init-from-fp)
   (:args (object :scs (descriptor-reg)))
