@@ -126,7 +126,11 @@ extern void scavenge_control_stack(struct thread *th);
 extern void scrub_control_stack(void);
 extern void scrub_thread_control_stack(struct thread *);
 
-#ifdef LISP_FEATURE_IMMOBILE_SPACE
+#ifndef LISP_FEATURE_IMMOBILE_SPACE
+
+static inline boolean filler_obj_p(lispobj* obj) { return 0; }
+
+#else
 
 extern void enliven_immobile_obj(lispobj*,int);
 extern void fixup_immobile_refs(lispobj (*)(lispobj), lispobj, struct code*);
@@ -138,25 +142,7 @@ extern void fixup_immobile_refs(lispobj (*)(lispobj), lispobj, struct code*);
 // because a simple-fun header does not contain a generation.
 #define __immobile_obj_generation(x) (__immobile_obj_gen_bits(x) & IMMOBILE_OBJ_GENERATION_MASK)
 
-static inline boolean immobile_space_p(lispobj obj)
-{
-  return IMMOBILE_SPACE_START <= obj && obj < IMMOBILE_SPACE_END;
-}
-
 typedef int low_page_index_t;
-static inline low_page_index_t find_immobile_page_index(void *addr)
-{
-  if (addr >= (void*)IMMOBILE_SPACE_START) {
-      // Must use full register size here to avoid truncation of quotient
-      // and bogus result!
-      page_index_t index =
-          ((uintptr_t)addr -
-           (uintptr_t)IMMOBILE_SPACE_START) / IMMOBILE_CARD_BYTES;
-      if (index < (int)(IMMOBILE_SPACE_SIZE/IMMOBILE_CARD_BYTES))
-          return index;
-  }
-  return -1;
-}
 
 #ifdef LISP_FEATURE_LITTLE_ENDIAN
 static inline int immobile_obj_gen_bits(lispobj* pointer) // native pointer
@@ -175,14 +161,10 @@ static inline int __immobile_obj_gen_bits(lispobj* pointer) // native pointer
 #error "Need to define immobile_obj_gen_bits() for big-endian"
 #endif /* little-endian */
 
-static inline boolean immobile_filler_p(lispobj* obj) {
+static inline boolean filler_obj_p(lispobj* obj) {
   return *(int*)obj == (2<<N_WIDETAG_BITS | CODE_HEADER_WIDETAG);
 }
 
-#else
-
-static inline boolean immobile_filler_p(lispobj* obj) { return 0; }
-
 #endif /* immobile space */
 
 #define WEAK_POINTER_NWORDS \
@@ -250,8 +232,9 @@ static inline boolean layout_bitmap_logbitp(int index, lispobj bitmap)
 static inline void unprotect_page_index(page_index_t page_index)
 {
     os_protect(page_address(page_index), GENCGC_CARD_BYTES, OS_VM_PROT_ALL);
-    page_table[page_index].write_protected_cleared = 1;
-    page_table[page_index].write_protected = 0;
+    unsigned char *pflagbits = (unsigned char*)&page_table[page_index].gen - 1;
+    __sync_fetch_and_or(pflagbits, WP_CLEARED_BIT);
+    __sync_fetch_and_and(pflagbits, ~WRITE_PROTECTED_BIT);
 }
 
 static inline void protect_page(void* page_addr, page_index_t page_index)
@@ -282,4 +265,25 @@ static inline void protect_page(void* page_addr, page_index_t page_index)
 
 #endif
 
-#endif /* _GC_INTERNAL_H_ */
+// For x86[-64], a simple-fun or closure's "self" slot is a fixum
+// On other backends, it is a lisp ointer.
+#if defined(LISP_FEATURE_X86) || defined(LISP_FEATURE_X86_64)
+#define FUN_SELF_FIXNUM_TAGGED 1
+#else
+#define FUN_SELF_FIXNUM_TAGGED 0
+#endif
+
+#ifdef LISP_FEATURE_IMMOBILE_SPACE
+static inline void *
+fixedobj_page_address(low_page_index_t page_num)
+{
+    return (void*)(FIXEDOBJ_SPACE_START + (page_num * IMMOBILE_CARD_BYTES));
+}
+static inline void *
+varyobj_page_address(low_page_index_t page_num)
+{
+    return (void*)(VARYOBJ_SPACE_START + (page_num * IMMOBILE_CARD_BYTES));
+}
+#endif
+
+#endif /* _GC_PRIVATE_H_ */
