@@ -139,6 +139,13 @@ struct page {
     generation_index_t gen;
 };
 extern struct page *page_table;
+#ifdef LISP_FEATURE_BIG_ENDIAN
+#define WRITE_PROTECTED_BIT (1<<4)
+#define WP_CLEARED_BIT (1<<3)
+#else
+#define WRITE_PROTECTED_BIT (1<<3)
+#define WP_CLEARED_BIT (1<<4)
+#endif
 
 struct __attribute__((packed)) corefile_pte {
   uword_t sso; // scan start offset
@@ -268,7 +275,6 @@ struct fixedobj_page { // 12 bytes per page
 extern struct fixedobj_page *fixedobj_pages;
 #define fixedobj_page_obj_align(i) (fixedobj_pages[i].attr.parts.obj_align<<WORD_SHIFT)
 #define fixedobj_page_obj_size(i) fixedobj_pages[i].attr.parts.obj_size
-#define FIRST_VARYOBJ_PAGE (IMMOBILE_FIXEDOBJ_SUBSPACE_SIZE/(int)IMMOBILE_CARD_BYTES)
 #endif
 
 extern page_index_t last_free_page;
