@@ -326,14 +326,14 @@ save_to_filehandle(FILE *file, char *filename, lispobj init_function,
 #ifdef LISP_FEATURE_IMMOBILE_SPACE
     output_space(file,
                  IMMOBILE_FIXEDOBJ_CORE_SPACE_ID,
-                 (lispobj *)IMMOBILE_SPACE_START,
-                 immobile_fixedobj_free_pointer,
+                 (lispobj *)FIXEDOBJ_SPACE_START,
+                 fixedobj_free_pointer,
                  core_start_pos,
                  core_compression_level);
     output_space(file,
                  IMMOBILE_VARYOBJ_CORE_SPACE_ID,
-                 (lispobj *)IMMOBILE_VARYOBJ_SUBSPACE_START,
-                 immobile_space_free_pointer,
+                 (lispobj *)VARYOBJ_SPACE_START,
+                 varyobj_free_pointer,
                  core_start_pos,
                  core_compression_level);
 #endif
