@@ -206,7 +206,7 @@
       (assert (not (ctu:find-code-constants f :type 'sb-kernel:fdefn))))))
 
 (with-test (:name :layout-constants
-                  :skipped-on '(not (and :x86-64 :immobile-space)))
+                  :skipped-on (not (and :x86-64 :immobile-space)))
   (let ((addr-of-pathname-layout
          (write-to-string
           (sb-kernel:get-lisp-obj-address (sb-kernel:find-layout 'pathname))
@@ -222,7 +222,7 @@
         (incf count)))
     (assert (= count 2))))
 
-(with-test (:name :set-symbol-value-imm :skipped-on '(not :x86-64))
+(with-test (:name :set-symbol-value-imm :skipped-on (not :x86-64))
   (let (success)
     (dolist (line (split-string
                    (with-output-to-string (s)
@@ -235,7 +235,7 @@
         (setq success t)))
     (assert success)))
 
-(with-test (:name :linkage-table-bogosity :skipped-on '(not :sb-dynamic-core))
+(with-test (:name :linkage-table-bogosity :skipped-on (not :sb-dynamic-core))
   (let ((strings (map 'list (lambda (x) (if (consp x) (car x) x))
                       #+sb-dynamic-core sb-vm::+required-foreign-symbols+
                       #-sb-dynamic-core '())))
@@ -324,6 +324,22 @@
   (compiles-with-warning `(lambda (x)
                             (find-if #'evenp (the string x)))))
 
+(with-test (:name (:type-conflict map-into :let))
+  (compiles-with-warning `(lambda (z)
+                            (let ((x "abc"))
+                              (map-into z #'evenp x)))))
+
+(with-test (:name (:type-conflict map-into :result))
+  (compiles-with-warning `(lambda (z)
+                            (map-into (make-string 10) #'evenp z))))
+
+(with-test (:name (:type-conflict complement))
+  (assert (nth-value 3
+                     (checked-compile
+                      `(lambda (z)
+                         (find z "l" :test (complement #'=)))
+                      :allow-style-warnings t))))
+
 (with-test (:name :type-across-hairy-lambda-transforms)
   (assert (subtypep (sb-kernel:%simple-fun-type (lambda (x) (find 1 (the vector x))))
                     '(function * (values (or (integer 1 1) null) &optional)))))
@@ -437,3 +453,198 @@
                 (list 3 lv1))
          (setf lv1 10)))
     (() 10)))
+
+(with-test (:name :transform-on-a-nil-arg)
+  (checked-compile-and-assert ()
+   `(lambda ()
+      (block nil
+        (logtest
+         (multiple-value-prog1
+             (unwind-protect (return 32))
+           (catch 'tag (return 33)))
+         1)
+        34))
+   (() 32)))
+
+(with-test (:name :nesteted-dx-deleted-uses)
+  (checked-compile-and-assert ()
+    `(lambda (a)
+       (block b2
+         (let* ((v1 (make-array nil :initial-element
+                                (let ((a a))
+                                  (return-from b2 a)))))
+           (declare (dynamic-extent v1))
+           (aref v1))))
+    ((342) 342)))
+
+(with-test (:name :deleted-during-locall-analyze-fun-1)
+  (checked-compile-and-assert (:allow-warnings t)
+    `(lambda ()
+       (flet ((a ()))
+         (a 1)
+         (a 2)))
+    (() (condition 'program-error))))
+
+(with-test (:name :delete-return-without-flush-dest)
+  (assert (eql
+           (catch 'c
+             (funcall (checked-compile
+                       '(lambda ()
+                         (labels ((%f () 40))
+                           (multiple-value-prog1 *
+                             (throw 'c (%f))
+                             (%f)
+                             30))))))
+           40)))
+
+(with-test (:name :let-conversion-inside-deleted-lambda.1)
+  (checked-compile-and-assert ()
+    `(lambda ()
+       (block nil
+         (catch 'c)
+         (flet ((f (x &key)
+                  (when x
+                    (progv '(*) '(0)
+                      (return)))))
+           (f (return 123))
+           (f 0))))
+    (() 123)))
+
+(with-test (:name :let-conversion-inside-deleted-lambda.2)
+  (checked-compile-and-assert ()
+    `(lambda ()
+       (block nil
+         (block nil
+           (lambda () (return)))
+         (labels ((l () (l))
+                  (%f (a &key)
+                    (l)
+                    (return a)))
+           (%f (return 321))
+           (%f 1))))
+    (() 321)))
+
+(with-test (:name :unconvert-tail-calls)
+  (checked-compile-and-assert ()
+    `(lambda ()
+       (block nil
+         (labels ((f (&optional (a (return))
+                                (b (if t (return)))
+                                c
+                      &rest args)
+                    (declare (ignore a b c args))
+                    (return 0)))
+           (let (x)
+             (equal 10 (f 0 3))
+             (f 123 0 0)
+             (f 0)
+             x))))
+    (() 0)))
+
+(with-test (:name :deleting-exits-with-multiple-users)
+  (checked-compile-and-assert ()
+    `(lambda (a b)
+       (block nil
+         (multiple-value-prog1 b
+           (tagbody (return (multiple-value-prog1 3
+                              (if a (go z)))) z))))
+    ((nil :good) 3)
+    ((t :good) :good)))
+
+(with-test (:name :merge-tail-sets-deleted-functional)
+  (checked-compile-and-assert ()
+    `(lambda (a)
+       (block nil
+         (tagbody
+            (go g549)
+          g549
+            (return-from nil
+              (block b3
+                (let ((x (progn (lambda (&optional (x a)) x)
+                                (unwind-protect 10)
+                                (return-from b3 a))))
+                  (unwind-protect x)))))))
+    ((321) 321)))
+
+(with-test (:name :interval-div-zero)
+  (checked-compile-and-assert (:optimize :safe)
+    `(lambda (x y)
+       (truncate (the (integer 0 0) x)
+                 (the (rational (1) (2)) y)))
+   ((0 3/2) (values 0 0))))
+
+(with-test (:name :float-remainders-rounding-errors)
+  (loop for fun in '(ceiling truncate floor
+                     fceiling ftruncate ffloor
+                     round fround)
+        do
+        (assert (member (second
+                         (third (sb-kernel:%simple-fun-type
+                                 (checked-compile
+                                  `(lambda (x)
+                                     (nth-value 1 (,fun (the double-float x) 1/2)))))))
+                        '(double-float real)))))
+
+(with-test (:name :float-quotient-rounding-errors)
+  (checked-compile-and-assert (:optimize :safe)
+   `(lambda ()
+      (floor -114658225103614 84619.58))
+    (() (values -1354984705 8473228.0)))
+  (checked-compile-and-assert (:optimize :safe)
+   `(lambda ()
+      (ceiling 114658225103614 84619.58))
+    (() (values 1354984705 -8473228.0))))
+
+(with-test (:name :complex-float-contagion)
+  (checked-compile-and-assert ()
+    `(lambda (p1)
+       (declare (type (or double-float integer) p1))
+       (complex p1 2.0))
+    ((1d0) #c(1d0 2d0))))
+
+(with-test (:name :equal-transform-member-types)
+  (let* ((s1 "abc")
+         (s2 (copy-seq s1)))
+    (checked-compile-and-assert ()
+      `(lambda (p1 p2)
+         (declare (type (member ,s1) p1)
+                  (type (member ,s2 #*10) p2))
+         (equal p1 p2))
+      ((s1 s2) t))))
+
+(with-test (:name :equalp-transform-numeric-types)
+  (checked-compile-and-assert ()
+    `(lambda (p1 p2)
+       (declare (type (or fixnum list) p1)
+                (type double-float p2))
+       (equalp p1 p2))
+    ((1 1d0) t)))
+
+(with-test (:name :equalp-transform-zero-array)
+  (checked-compile-and-assert ()
+    `(lambda (a b)
+       (declare (simple-string a)
+                (simple-bit-vector b))
+       (equalp a b))
+    (("" #*) t)))
+
+(with-test (:name :fill-transform-returning-array-data)
+  (let ((vector (make-array 10 :fill-pointer 2)))
+    (checked-compile-and-assert ()
+      `(lambda (v)
+           (declare (type (vector t) v))
+           (fill v nil))
+      ((vector) vector))))
+
+(with-test (:name :missing-error-context)
+  (flet ((run ()
+          (let ((string
+                 (with-output-to-string (*error-output*)
+                   (compile nil '(sb-int:named-lambda bob () (otherfun) 3)))))
+            (assert (search "in: SB-INT:NAMED-LAMBDA BOB" string)))))
+    (run)
+    ;; Unrepeatability is confusing:
+    ;; The first compiler invocation used to leave *last-format-string*
+    ;; with a toplevel value, so the second would not print enough context
+    ;; because the format control and args were the same.
+    (run)))
