@@ -101,20 +101,17 @@
     (assert (search "returned OK" output))))
 
 ;;; bug 379
-;;; This is not a WITH-TEST :FAILS-ON PPC DARWIN since there are
-;;; suspicions that the breakpoint trace might corrupt the whole image
-;;; on that platform.
 (with-test (:name (trace :encapsulate nil)
-            :fails-on '(or (and :ppc (not :linux)) :sparc :arm64)
-            :broken-on '(or :darwin :sunos :hppa))
+            :fails-on (or (and :ppc (not :linux)) :sparc :arm64)
+            :broken-on (or :sunos :hppa))
   (let ((output (with-traced-function (trace-this :encapsulate nil)
                   (assert (eq 'ok (trace-this))))))
     (assert (search "TRACE-THIS" output))
     (assert (search "returned OK" output))))
 
 (with-test (:name (:trace :encapsulate nil :recursive)
-            :fails-on '(or (and :ppc (not :linux)) :sparc :sunos :arm64)
-            :broken-on '(or :darwin (and :x86 :sunos) :hppa))
+            :fails-on (or (and :ppc (not :linux)) :sparc :sunos :arm64)
+            :broken-on (or (and :x86 :sunos) :hppa))
   (let ((output (with-traced-function (trace-fact :encapsulate nil)
                   (assert (= 120 (trace-fact 5))))))
     (assert (search "TRACE-FACT" output))
@@ -166,7 +163,7 @@
     (untrace)
     (assert (>= (count #\Newline (get-output-stream-string s)) 4))))
 
-(with-test (:name :bug-310175 :fails-on '(not :stack-allocatable-lists))
+(with-test (:name :bug-310175 :fails-on (not :stack-allocatable-lists))
   ;; KLUDGE: Not all DX-enabled platforms DX CONS, and the compiler
   ;; transforms two-arg-LIST* (and one-arg-LIST) to CONS.  Therefore,
   ;; use two-arg-LIST, which should get through to VOP LIST, and thus
@@ -239,7 +236,7 @@
   (test-infinite-error-protection))
 
 (with-test (:name (:infinite-error-protection :thread)
-                  :skipped-on '(not :sb-thread))
+                  :skipped-on (not :sb-thread))
   (enable-debugger)
   (let ((thread (sb-thread:make-thread #'test-infinite-error-protection)))
     (loop while (sb-thread:thread-alive-p thread))))
