@@ -19,7 +19,7 @@
 
 ;;;; properties of symbols, e.g. presence of doc strings for public symbols
 
-(with-test (:name (documentation :cl) :skipped-on '(:not :sb-doc))
+(with-test (:name (documentation :cl) :skipped-on (:not :sb-doc))
   (let ((n 0))
     (do-symbols (s 'cl)
       (if (fboundp s)
@@ -79,7 +79,7 @@
 
 ;;; SLEEP should not cons except on 32-bit platforms when
 ;;; (> (mod seconds 1) (* most-positive-fixnum 1e-9))
-(with-test (:name (sleep :non-consing) :fails-on :win32
+(with-test (:name (sleep :non-consing)
                   :skipped-on :interpreter)
   (handler-case (sb-ext:with-timeout 5
                   (ctu:assert-no-consing (sleep 0.00001s0))
@@ -229,7 +229,7 @@
 ;;; This tests that the x86-64 disasembler does not crash
 ;;; on LEA with a rip-relative operand and no label.
 (with-test (:name (disassemble :no-labels)
-                  :skipped-on '(not :x86-64))
+                  :skipped-on (not :x86-64))
   (let* ((lines
           (split-string
            (with-output-to-string (stream)
