@@ -298,7 +298,7 @@
                (frob pathname-type)))))
 
 ;;; host-namestring testing
-(with-test (:name :host-namestring)
+(with-test (:name host-namestring)
   (assert (string=
            (namestring (parse-namestring "/foo" (host-namestring #p"/bar")))
            "/foo"))
@@ -338,7 +338,7 @@
                    (make-pathname :name "foo" :type "txt" :version 1)
                    (make-pathname :name "foo" :type ".txt")
                    (make-pathname :name "foo." :type "txt")
-                   (make-pathname :name "\\" :type "txt")
+                   (make-pathname :name #-win32 "\\" #+win32 "^^" :type "txt")
                    (make-pathname :name "^" :type "txt")
                    (make-pathname :name "foo*" :type "txt")
                    (make-pathname :name "foo[" :type "txt")
@@ -384,7 +384,7 @@
 
 ;;; bug reported by Artem V. Andreev: :WILD not handled in unparsing
 ;;; directory lists.
-(with-test (:name :unparse-wild)
+(with-test (:name (namestring :unparse-wild))
   (assert (equal (namestring #p"/tmp/*/") "/tmp/*/")))
 
 ;;; Printing of pathnames; see CLHS 22.1.3.1. This section was started
@@ -414,11 +414,20 @@
 
 ;;; we failed to unparse logical pathnames with :NAME :WILD :TYPE NIL.
 ;;; (Reported by Pascal Bourguignon.
-(with-test (:name :unparse-logical-wild)
+(with-test (:name (namestring :unparse-logical-wild))
   (let ((pathname (make-pathname :host "SYS" :directory '(:absolute :wild-inferiors)
                                  :name :wild :type nil)))
     (assert (string= (namestring pathname) "SYS:**;*"))
     (assert (string= (write-to-string pathname :readably t) "#P\"SYS:**;*\""))))
+
+(with-test (:name (namestring :signals file-error))
+  (flet ((test (&rest initargs)
+           (let ((pathname (apply #'make-pathname initargs)))
+             (assert-error (namestring pathname) file-error))))
+    (test :name "foo.")
+    (test :name "")
+    (test :type "foo")
+    (test :name "foo" :type ".bar")))
 
 ;;; reported by James Y Knight on sbcl-devel 2006-05-17
 (with-test (:name :merge-back)
@@ -427,6 +436,16 @@
     (assert (equal (merge-pathnames p1 p2)
                    (make-pathname :directory '(:relative :back "foo" "bar"))))))
 
+(with-test (:name (native-namestring :signals file-error :unix))
+  (flet ((test (&rest initargs)
+           (let ((pathname (apply #'make-pathname initargs)))
+             (assert-error (native-namestring pathname) file-error))))
+    (test :directory '(:absolute (:home "no-such-user")))
+    (test :directory '(:absolute :wild))
+    (test :name :wild)
+    (test :name "foo" :type :wild)
+    (test :type "bar")))
+
 ;;; construct native namestrings even if the directory is empty (means
 ;;; that same as if (:relative))
 (with-test (:name (sb-ext:native-namestring 1))
@@ -532,7 +551,7 @@
 ;;; Reported by Willem Broekema: Reading #p"\\\\" caused an error due
 ;;; to insufficient sanity in input testing in EXTRACT-DEVICE (in
 ;;; src;code;win32-pathname).
-(with-test (:name :bug-489698 :skipped-on '(not :win32))
+(with-test (:name :bug-489698 :skipped-on (not :win32))
   (assert (equal (make-pathname :directory '(:absolute))
                  (read-from-string "#p\"\\\\\\\\\""))))
 
@@ -586,10 +605,15 @@
 
 ;;; lp#673625
 (with-test (:name :pathname-escape-first-directory-component
-                  :fails-on :win32)
+            :fails-on :win32)
   ;; ~ / :HOME
-  (assert (equal (pathname-directory #p"\\~/foo/") '(:relative "~" "foo")))
-  (assert (equal (native-namestring #p"\\~/foo/") "~/foo/"))
+  (assert (equal (pathname-directory #-win32 #p"\\~/foo/"
+                                     #+win32 #p"^~/foo/")
+                 '(:relative "~" "foo")))
+  (assert (equal (native-namestring #-win32 #p"\\~/foo/"
+                                    #+win32 #p"^~/foo/")
+                 #-win32 "~/foo/"
+                 #+win32 "~\\foo\\"))
   (assert (equal (namestring (make-pathname :directory '(:absolute "~zot")))
                  "\\~zot/"))
   ;; * / :WILD
@@ -599,7 +623,7 @@
   (let ((*default-pathname-defaults* #p"/tmp/foo"))
     (ensure-directories-exist "/")))
 
-(with-test (:name :long-file-name :skipped-on '(not :win32))
+(with-test (:name :long-file-name :skipped-on (not :win32))
   (let* ((x '("hint--if-you-are-having-trouble-deleting-this-test-directory"
               "use-the-7zip-file-manager"))
          (base (truename
