@@ -19,10 +19,9 @@
 ;; (sometimes the handler will manage to WAIT3 a process before
 ;; run-tests WAITPIDs it).
 
-(with-test (:name :run-program-cat-1)
-  (let* ((process (sb-ext:run-program "cat" '() :wait nil
-                                      :search t
-                                      :output :stream :input :stream))
+(with-test (:name (run-program :cat 1))
+  (let* ((process (run-program "cat" '() :wait nil
+                               :search t :output :stream :input :stream))
          (out (process-input process))
          (in (process-output process)))
     (unwind-protect
@@ -32,11 +31,11 @@
               (assert (= (read-byte in) i)))
       (process-close process))))
 
-(with-test (:name :run-program-cat-2 :skipped-on '(or (not :sb-thread) :win32))
+(with-test (:name (run-program :cat 2)
+                  :skipped-on (or (not :sb-thread) :win32))
   ;; Tests that reading from a FIFO is interruptible.
-  (let* ((process (sb-ext:run-program "/bin/cat" '()
-                                      :wait nil
-                                      :output :stream :input :stream))
+  (let* ((process (run-program "/bin/cat" '()
+                               :wait nil :output :stream :input :stream))
          (in (process-input process))
          (out (process-output process))
          (sem (sb-thread:make-semaphore))
@@ -98,28 +97,28 @@
              :start2 start :end2 end)
     seq))
 
-(with-test (:name :run-program-cat-3)
+(with-test (:name (run-program :cat 3))
   ;; User-defined binary input and output streams.
   (let ((in (make-instance 'buffer-stream))
         (out (make-instance 'buffer-stream))
         (data #(0 1 2 3 4 5 6 7 8 9 10 11 12)))
     (write-sequence data in)
-    (let ((process (sb-ext:run-program "cat" '()
-                                       :search t
-                                       :wait t
-                                       :output out :input in))
+    (let ((process (run-program "cat" '()
+                                :search t
+                                :wait t
+                                :output out :input in))
           (buf (make-array (length data))))
       (declare (ignore process))
       (assert (= 13 (read-sequence buf out)))
       (assert (= 0 (read-sequence (make-array 8) out)))
       (assert (equalp buf data)))))
 
-(with-test (:name :run-program-cat-4)
+(with-test (:name (run-program :cat 4))
   ;; Null broadcast stream as output
-  (let* ((process (sb-ext:run-program "cat" '() :wait nil
-                                      :search t
-                                      :output (make-broadcast-stream)
-                                      :input :stream))
+  (let* ((process (run-program "cat" '() :wait nil
+                               :search t
+                               :output (make-broadcast-stream)
+                               :input :stream))
          (in (process-input process)))
     (unwind-protect
          (progn
@@ -152,7 +151,7 @@
   (defparameter *cat-out-pipe* (make-pipe))
   (defparameter *cat-out* (make-synonym-stream '*cat-out-pipe*)))
 
-(with-test (:name :run-program-cat-5 :fails-on :win32)
+(with-test (:name (run-program :cat 5) :fails-on :win32)
   (let ((cat (run-program "/bin/cat" nil :input *cat-in* :output *cat-out*
                           :wait nil)))
     (dolist (test '("This is a test!"
@@ -174,7 +173,7 @@
 (progn
   (defparameter *tmpfile* "run-program-ed-test.tmp")
 
-  (with-test (:name :run-program-/bin/ed)
+  (with-test (:name (run-program :/bin/ed))
     (with-open-file (f *tmpfile*
                        :direction :output
                        :if-exists :supersede)
@@ -211,34 +210,33 @@
 ;; Around 1.0.12 there was a regression when :INPUT or :OUTPUT was a
 ;; pathname designator.  Since these use the same code, it should
 ;; suffice to test just :INPUT.
-(let ((file))
-  (unwind-protect
-       (progn (with-open-file (f "run-program-test.tmp" :direction :output)
-                (setf file (truename f))
-                (write-line "Foo" f))
-                  (assert (run-program "cat" ()
-                                       :input file :output t
-                                       :search t :wait t)))
-    (when file
-      (delete-file file))))
+(with-test (:name (run-program :input :output pathname))
+  (let ((file))
+    (unwind-protect
+         (progn (with-open-file (f "run-program-test.tmp" :direction :output)
+                  (setf file (truename f))
+                  (write-line "Foo" f))
+                (assert (run-program "cat" ()
+                                     :input file :output t
+                                     :search t :wait t)))
+      (when file
+        (delete-file file)))))
 
 ;;; This used to crash on Darwin and trigger recursive lock errors on
 ;;; every platform.
-(with-test (:name (:run-program :stress))
+(with-test (:name (run-program :stress))
   ;; Do it a hundred times in batches of 10 so that with a low limit
   ;; of the number of processes the test can have a chance to pass.
   (loop
    repeat 10 do
-   (map nil
-        #'sb-ext:process-wait
+   (map nil #'process-wait
         (loop repeat 10
               collect
-              (sb-ext:run-program "echo"
-                                  '("It would be nice if this didn't crash.")
-                                  :search t
-                                  :wait nil :output nil)))))
+              (run-program "echo"
+                           '("It would be nice if this didn't crash.")
+                           :search t :wait nil :output nil)))))
 
-(with-test (:name (:run-program :pty-stream) :fails-on :win32)
+(with-test (:name (run-program :pty-stream) :fails-on :win32)
   (assert (equal "OK"
                  (handler-case
                   (with-timeout 1
@@ -258,14 +256,14 @@
 ;; We can't check for the signal itself since run-program.c resets the
 ;; forked process' signal mask to defaults. But the default is `stop'
 ;; of which we can be notified asynchronously by providing a status hook.
-(with-test (:name (:run-program :inherit-stdin) :fails-on :win32)
+(with-test (:name (run-program :inherit-stdin) :fails-on :win32)
   (let (stopped)
     (flet ((status-hook (proc)
-             (case (sb-ext:process-status proc)
+             (case (process-status proc)
                (:stopped (setf stopped t)))))
-      (let ((proc (sb-ext:run-program "/bin/ed" nil :search nil :wait nil
-                                      :input t :output t
-                                      :status-hook #'status-hook)))
+      (let ((proc (run-program "/bin/ed" nil :search nil :wait nil
+                               :input t :output t
+                               :status-hook #'status-hook)))
         ;; Give the program a generous time to generate the SIGTTIN.
         ;; If it hasn't done so after that time we can consider it
         ;; to be working (i.e. waiting for input without generating SIGTTIN).
@@ -278,7 +276,7 @@
 
 ;; Check that in when you do run-program with :wait t that causes
 ;; encoding error, it does not affect the following run-program
-(with-test (:name (:run-program :clean-exit-after-encoding-error)
+(with-test (:name (run-program :clean-exit-after-encoding-error)
                   :fails-on :win32)
   (let ((had-error-p nil))
     (flet ((barf (&optional (format :default))
@@ -305,14 +303,14 @@
           (setq had-error-p t)))
       (assert (not had-error-p)))))
 
-(with-test (:name (:run-program :no-such-thing))
+(with-test (:name (run-program :no-such-thing))
   (assert (search "Couldn't execute"
                   (handler-case
                       (progn (run-program "no-such-program-we-hope" '()) nil)
                     (error (e)
                       (princ-to-string e))))))
 
-(with-test (:name (:run-program :not-executable))
+(with-test (:name (run-program :not-executable))
   (assert (search "Couldn't execute"
                   (handler-case
                       (progn (run-program "run-program.impure.lisp" '()) nil)
@@ -320,57 +318,56 @@
                       (princ-to-string e))))))
 
 #-win32
-(with-test (:name (:run-program :if-input-does-not-exist))
+(with-test (:name (run-program :if-input-does-not-exist))
   (let ((file (pathname (sb-posix:mktemp "rpXXXXXX"))))
-    (assert (null (sb-ext:run-program "/bin/cat" '() :input file)))
-    (assert (null (sb-ext:run-program "/bin/cat" '() :output #.(or *compile-file-truename*
-                                                                   *load-truename*)
+    (assert (null (run-program "/bin/cat" '() :input file)))
+    (assert (null (run-program "/bin/cat" '() :output #.(or *compile-file-truename*
+                                                            *load-truename*)
                                       :if-output-exists nil)))))
 
 
-(with-test (:name (:run-program :set-directory))
+(with-test (:name (run-program :set-directory))
   (let* ((directory #-win32 "/"
                     #+win32 "c:\\")
-         (out (sb-ext:process-output
-               (sb-ext:run-program #-win32 "/bin/sh"
-                                   #-win32 '("-c" "pwd")
-                                   #+win32 "cmd.exe"
-                                   #+win32 '("/c" "cd")
-                                   :output :stream
-                                   :directory directory
-                                   :search t))))
+         (out (process-output
+               (run-program #-win32 "/bin/sh"
+                            #-win32 '("-c" "pwd")
+                            #+win32 "cmd.exe"
+                            #+win32 '("/c" "cd")
+                            :output :stream
+                            :directory directory
+                            :search t))))
     (assert
      (equal directory
             (string-right-trim '(#\Return) (read-line out))))))
 
-(with-test (:name (:run-program :directory-nil))
-  (sb-ext:run-program #-win32 "/bin/sh"
-                      #-win32 '("-c" "pwd")
-                      #+win32 "cmd.exe"
-                      #+win32 '("/c" "cd")
-                      :directory nil
-                      :search t))
+(with-test (:name (run-program :directory-nil))
+  (run-program #-win32 "/bin/sh"
+               #-win32 '("-c" "pwd")
+               #+win32 "cmd.exe"
+               #+win32 '("/c" "cd")
+               :directory nil
+               :search t))
 
-(with-test (:name (:run-program :bad-options))
+(with-test (:name (run-program :bad-options))
   (assert-error
-   (sb-ext:run-program #-win32 "/bin/sh"
-                       #-win32 '("-c" "pwd")
-                       #+win32 "cmd.exe"
-                       #+win32 '("/c" "cd")
-                       :search t
-                       :output :bad)))
+   (run-program #-win32 "/bin/sh"
+                #-win32 '("-c" "pwd")
+                #+win32 "cmd.exe"
+                #+win32 '("/c" "cd")
+                :search t
+                :output :bad)))
 
-(with-test (:name (sb-ext:run-program :stop+continue) :skipped-on (not :linux))
-  (let ((process (sb-ext:run-program "cat" '()
-                                     :search t :input :stream :wait nil)))
+(with-test (:name (run-program :stop+continue) :skipped-on :win32)
+  (let ((process (run-program "cat" '() :search t :input :stream :wait nil)))
     (flet ((kill-and-check-status (signal expected-status)
-             (sb-ext:process-kill process signal)
-             (loop :repeat 500
-                :when (eq (sb-ext:process-status process) expected-status)
+             (process-kill process signal)
+             (loop :repeat 2000
+                :when (eq (process-status process) expected-status)
                 :do (return)
                 :do (sleep 1/100)
                 :finally (error "~@<Process ~A did not change its ~
-                                 status to ~S within five seconds.~@:>"
+                                 status to ~S within 20 seconds.~@:>"
                                 process expected-status))))
       (kill-and-check-status sb-posix:sigstop :stopped)
       (kill-and-check-status sb-posix:sigcont :continued)
