@@ -39,6 +39,7 @@
 #+sb-thread
 (defun make-kill-thread (&rest args)
   (let ((thread (apply #'sb-thread:make-thread args)))
+    #-win32 ;; poor thread interruption on safepoints
     (when (boundp '*threads-to-kill*)
       (push thread *threads-to-kill*))
     thread))
@@ -76,9 +77,9 @@
     `(progn
        (start-test)
        (cond
-         ((broken-p ,broken-on)
+         ((broken-p ',broken-on)
           (fail-test :skipped-broken ',name "Test broken on this platform"))
-         ((skipped-p ,skipped-on)
+         ((skipped-p ',skipped-on)
           (fail-test :skipped-disabled ',name "Test disabled for this combination of platform and features"))
          (t
           (let (#+sb-thread (,threads (sb-thread:list-all-threads))
@@ -86,7 +87,7 @@
                 (*threads-to-kill* nil))
             (block ,block-name
               (handler-bind ((error (lambda (error)
-                                      (if (expected-failure-p ,fails-on)
+                                      (if (expected-failure-p ',fails-on)
                                           (fail-test :expected-failure ',name error)
                                           (fail-test :unexpected-failure ',name error))
                                       (return-from ,block-name))))
@@ -114,11 +115,12 @@
                                   (member thread ,threads)
                                   (sb-thread:thread-ephemeral-p thread))
                         (setf any-leftover thread)
+                        #-win32
                         (ignore-errors (sb-thread:terminate-thread thread))))
                     (when any-leftover
                       (fail-test :leftover-thread ',name any-leftover)
                       (return-from ,block-name)))
-                  (if (expected-failure-p ,fails-on)
+                  (if (expected-failure-p ',fails-on)
                       (fail-test :unexpected-success ',name nil)
                       ;; Non-pretty is for cases like (with-test (:name (let ...)) ...
                       (log-msg/non-pretty "Success ~S" ',name)))))))))))
@@ -302,17 +304,20 @@
   (cond
     ((functionp thing)
      (error "~@<~S is a function, not a form.~@:>" thing))
-    ((and optimize (not (typep thing '(cons (eql lambda)))))
-     (error "~@<Cannot splice ~A declaration into forms other than ~
-             ~S: ~S.~@:>"
-            'optimize 'lambda thing))
-    (optimize
+    ((not optimize)
+     thing)
+    ((typep thing '(cons (eql sb-int:named-lambda)))
+     `(,@(subseq thing 0 3)
+         (declare (optimize ,@optimize))
+         ,@(nthcdr 3 thing)))
+    ((typep thing '(cons (eql lambda)))
      `(,(first thing) ,(second thing)
         (declare (optimize ,@optimize))
         ,@(nthcdr 2 thing)))
-
     (t
-     thing)))
+     (error "~@<Cannot splice ~A declaration into forms other than ~
+             ~{~S~#[~; and ~:;, ~]~}: ~S.~@:>"
+            'optimize '(lambda sb-int:named-lambda) thing))))
 
 (defun compile-capturing-output-and-conditions
     (form &key name condition-transform)
@@ -557,7 +562,7 @@
 ;;;   (CONDITION CONDITION-TYPE)
 ;;;
 ;;; the function call is expected to signal the designated condition
-;;; instead of returning values.
+;;; instead of returning values. CONDITION-TYPE is evaluated.
 ;;;
 ;;; The OPTIMIZE keyword parameter controls the optimization policies
 ;;; (or policy) used when compiling FORM. The argument is interpreted
@@ -582,7 +587,7 @@
                           values :test test)))
                `(list (lambda () (values ,@args))
                       ,(if conditionp
-                           `',values
+                           `(list 'condition ,(second values))
                            `(lambda () (multiple-value-list ,values)))
                       ,test)))))
     `(%checked-compile-and-assert
